@@ -123,13 +123,15 @@ log = logging.getLogger()
 # if we are *not* in setup mode, import the logger and possibly populate the
 # configuration file with the defaults
 if not _ASTROPY_SETUP_:
-    from .logger import _init_log
+    from .logger import _init_log, _teardown_log
     from . import config
 
     import os
     import sys
     from warnings import warn
 
+    log = _init_log()
+
     try:
         from .utils import _compiler
     except ImportError:
@@ -138,19 +140,25 @@ if not _ASTROPY_SETUP_:
                       'a source checkout; please run `./setup.py develop` or '
                       '`./setup.py build_ext --inplace` first so that '
                       'extension modules can be compiled and made importable.')
-            sys.exit(1)
+            # Now disable exception logging to avoid an annoying error in the
+            # exception logger before we raise the import error:
+            _teardown_log()
+
+            # Roll back any astropy sub-modules that have been imported thus
+            # far
+
+            for key in sys.modules.keys():
+                if key.startswith('astropy.'):
+                    del sys.modules[key]
+            raise ImportError('astropy')
         else:
             # Outright broken installation; don't be nice.
             raise
 
-
-
     # add these here so we only need to cleanup the namespace at the end
     config_dir = None
-
-    log = _init_log()
-
     config_dir = os.path.dirname(__file__)
+
     try:
         config.configuration.update_default_config(__package__, config_dir)
     except config.configuration.ConfigurationDefaultMissingError as e:
