@@ -17,6 +17,7 @@ from contextlib import contextmanager
 
 from ..extern.configobj import configobj, validate
 
+
 __all__ = ['ConfigurationItem', 'InvalidConfigurationItemWarning',
            'ConfigurationMissingWarning', 'get_config', 'save_config',
            'reload_config']
@@ -299,6 +300,23 @@ class ConfigurationItem(object):
 
         baseobj[self.name] = newobj[self.name]
 
+    def __repr__(self):
+        out = '<{0}: name={1!r} value={2!r} at 0x{3:x}>'.format(
+            self.__class__.__name__, self.name, self(), id(self))
+        return out
+
+    def __str__(self):
+        out = '\n'.join(('{0}: {1}',
+                         '  cfgtype={2!r}',
+                         '  defaultvalue={3!r}',
+                         '  description={4!r}',
+                         '  module={5}',
+                         '  value={6!r}'))
+        out = out.format(self.__class__.__name__, self.name, self.cfgtype,
+                         self.defaultvalue, self.description, self.module,
+                         self())
+        return out
+
     def __call__(self):
         """ Returns the value of this `ConfigurationItem`
 
@@ -399,22 +417,25 @@ def get_config(packageormod=None, reload=False):
     cobj = _cfgobjs.get(rootname, None)
 
     if cobj is None:
-        try:
-            cfgfn = join(get_config_dir(), rootname + '.cfg')
-            cobj = configobj.ConfigObj(cfgfn, interpolation=False)
-            _cfgobjs[rootname] = cobj
-
-        except (IOError, OSError) as e:
-            msg1 = 'Configuration defaults will be used, and configuration '
-            msg2 = 'cannot be saved due to '
-            errstr = '' if len(e.args) < 1 else (':' + str(e.args[0]))
-            wmsg = msg1 + msg2 + e.__class__.__name__ + errstr
-            warn(ConfigurationMissingWarning(wmsg))
-
-            # This caches the object, so if the file becomes accessible, this
-            # function won't see it unless the module is reloaded
+        if _ASTROPY_SETUP_:
+            # There's no reason to use anything but the default config
             cobj = configobj.ConfigObj(interpolation=False)
-            _cfgobjs[rootname] = cobj
+        else:
+            try:
+                cfgfn = join(get_config_dir(), rootname + '.cfg')
+                cobj = configobj.ConfigObj(cfgfn, interpolation=False)
+            except (IOError, OSError) as e:
+                msg = ('Configuration defaults will be used, and '
+                       'configuration cannot be saved due to ')
+                errstr = '' if len(e.args) < 1 else (':' + str(e.args[0]))
+                msg += e.__class__.__name__ + errstr
+                warn(ConfigurationMissingWarning(msg))
+
+                # This caches the object, so if the file becomes accessible, this
+                # function won't see it unless the module is reloaded
+                cobj = configobj.ConfigObj(interpolation=False)
+
+        _cfgobjs[rootname] = cobj
 
     if secname:  # not the root package
         if secname not in cobj:
@@ -619,6 +640,8 @@ def generate_all_config_items(pkgornm=None, reset_to_default=False,
                              'have __file__ or __path__')
 
     for imper, nm, ispkg in walk_packages(pkgpath, package.__name__ + '.'):
+        if nm == 'astropy.config.tests.test_configs':
+            continue
         if not _unsafe_import_regex.match(nm):
             imper.find_module(nm)
             if reset_to_default:
