@@ -65,8 +65,11 @@ class _AngleParser(object):
         # NOTE THE ORDERING OF THESE RULES IS IMPORTANT!!
         # Regular expression rules for simple tokens
         def t_UFLOAT(t):
-            r'((\d+\.\d*)|(\.\d+))([eE][+-]?\d+)?'
-            t.value = float(t.value)
+            r'((\d+\.\d*)|(\.\d+))([eE][+-−]?\d+)?'
+            # The above includes Unicode "MINUS SIGN" \u2212.  It is
+            # important to include the hyphen last, or the regex will
+            # treat this as a range.
+            t.value = float(t.value.replace('−', '-'))
             return t
 
         def t_UINT(t):
@@ -75,8 +78,14 @@ class _AngleParser(object):
             return t
 
         def t_SIGN(t):
-            r'[+-]'
-            t.value = float(t.value + '1')
+            r'[+−-]'
+            # The above include Unicode "MINUS SIGN" \u2212.  It is
+            # important to include the hyphen last, or the regex will
+            # treat this as a range.
+            if t.value == '+':
+                t.value = 1.0
+            else:
+                t.value = -1.0
             return t
 
         t_COLON = ':'
@@ -248,7 +257,7 @@ def _check_hour_range(hrs):
     Checks that the given value is in the range (-24, 24).
     """
     if math.fabs(hrs) == 24.:
-        warn(IllegalHourWarning(hrs, 'Treating as 24 hr'))
+        warn(str(IllegalHourError(hrs)))
     elif not -24. < hrs < 24.:
         raise IllegalHourError(hrs)
 
@@ -259,7 +268,7 @@ def _check_minute_range(min):
     is equal to 60, then a warning is raised.
     """
     if min == 60.:
-        warn(IllegalMinuteWarning(min, 'Treating as 0 min, +1 hr/deg'))
+        warn(str(IllegalMinuteError(min)))
     elif not 0. <= min < 60.:
         # "Error: minutes not in range [0,60) ({0}).".format(min))
         raise IllegalMinuteError(min)
@@ -271,7 +280,7 @@ def _check_second_range(sec):
     is equal to 60, then a warning is raised.
     """
     if sec == 60.:
-        warn(IllegalSecondWarning(sec, 'Treating as 0 sec, +1 min'))
+        warn(str(IllegalSecondError(sec)))
     elif not 0. <= sec < 60.:
         # "Error: seconds not in range [0,60) ({0}).".format(sec))
         raise IllegalSecondError(sec)
@@ -514,7 +523,8 @@ def hours_to_string(h, precision=5, pad=False, sep=('h', 'm', 's')):
                '{sep[2]}')
     h, m, s = hours_to_hms(h)
     return literal.format(h, abs(m), abs(s), sep=sep, pad=pad,
-                          width=(precision + 3), precision=precision)
+                          width=(precision + 3 if precision > 0 else 2),
+                          precision=precision)
 
 
 def degrees_to_string(d, precision=5, pad=False, sep=':'):
@@ -546,7 +556,8 @@ def degrees_to_string(d, precision=5, pad=False, sep=':'):
                '{sep[2]}')
     d, m, s = degrees_to_dms(d)
     return literal.format(d, abs(m), abs(s), sep=sep, pad=pad,
-                          width=(precision + 3), precision=precision)
+                          width=(precision + 3 if precision > 0 else 2),
+                          precision=precision)
 
 
 #<----------Spherical angular distances------------->
