@@ -89,6 +89,11 @@ def test_create_angles():
 
     a15 = Angle("5h4m3s") # single digits, no decimal
 
+    a16 = Angle("1 d")
+    a17 = Angle("1 degree")
+    assert a16.degrees == 1
+    assert a17.degrees == 1
+
     #ensure the above angles that should match do
     a1 == a2 == a3 == a4 == a5 == a6 == a7
     npt.assert_almost_equal(a1.radians, a2.radians)
@@ -352,10 +357,10 @@ def test_angle_formatting():
     assert "Angle as DMS: {0}".format(angle.format(unit=u.degree, precision=4,
                                                   pad=True)) == res
 
-    res = 'Angle as rad: 0.062976radian'
+    res = 'Angle as rad: 0.06298radian'
     assert "Angle as rad: {0}".format(angle.format(unit=u.radian)) == res
 
-    res = 'Angle as rad decimal: 0.062976'
+    res = 'Angle as rad decimal: 0.06298'
     assert "Angle as rad decimal: {0}".format(angle.format(unit=u.radian, decimal=True)) == res
 
 
@@ -368,7 +373,8 @@ def test_angle_formatting():
     assert angle.format(pad=True) == '-01d14m04.44440s'
     assert angle.format(unit=u.hour) == '-0h04m56.29629s'
     assert angle2.format(unit=u.hour, pad=True) == '-01h14m04.44440s'
-    assert angle.format(unit=u.radian, decimal=True) == '-0.021547'
+    assert angle.format(unit=u.radian, decimal=True) == '-0.02155'
+
 
 def test_angle_format_roundtripping():
     """
