@@ -22,7 +22,7 @@ from . import parameters
 
 __all__ = ["FLRW", "LambdaCDM", "FlatLambdaCDM", "wCDM", "FlatwCDM",
            "Flatw0waCDM", "w0waCDM", "wpwaCDM", "w0wzCDM","get_current",
-           "set_current", "WMAP5", "WMAP7", "WMAP9"]
+           "set_current", "WMAP5", "WMAP7", "WMAP9", "Planck13"]
 
 # Constants
 
@@ -877,8 +877,8 @@ class FLRW(Cosmology):
 
         dh = self._hubble_distance
         dm = self.comoving_transverse_distance(z)
-        term1 = 4. * pi * dh**3 / (2. * Ok0)
-        term2 = dm / dh * sqrt(1 + Ok0 * (dm / dh)**2)
+        term1 = 4. * pi * dh ** 3 / (2. * Ok0)
+        term2 = dm / dh * np.sqrt(1 + Ok0 * (dm / dh) ** 2)
         term3 = sqrt(abs(Ok0)) * dm / dh
 
         if Ok0 > 0:
