@@ -1,4 +1,5 @@
-"""Asciitable: an extensible ASCII table reader and writer.
+# Licensed under a 3-clause BSD style license - see LICENSE.rst
+"""An extensible ASCII table reader and writer.
 
 latex.py:
   Classes to read and write LaTeX tables
@@ -7,7 +8,7 @@ latex.py:
 :Author: Tom Aldcroft (aldcroft@head.cfa.harvard.edu)
 """
 
-## 
+##
 ## Redistribution and use in source and binary forms, with or without
 ## modification, are permitted provided that the following conditions are met:
 ##     * Redistributions of source code must retain the above copyright
@@ -18,7 +19,7 @@ latex.py:
 ##     * Neither the name of the Smithsonian Astrophysical Observatory nor the
 ##       names of its contributors may be used to endorse or promote products
 ##       derived from this software without specific prior written permission.
-## 
+##
 ## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ## ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 ## WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -27,7 +28,7 @@ latex.py:
 ## (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 ## LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ## ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-## (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS  
+## (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ## SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 import re
@@ -36,18 +37,20 @@ from . import core
 latexdicts ={'AA':  {'tabletype': 'table',
                  'header_start': r'\hline \hline', 'header_end': r'\hline',
                  'data_end': r'\hline'},
-             'doublelines': {'tabletype': 'table', 
+             'doublelines': {'tabletype': 'table',
                  'header_start': r'\hline \hline', 'header_end': r'\hline\hline',
                  'data_end': r'\hline\hline'},
              'template': {'tabletype': 'tabletype', 'caption': 'caption',
-                 'col_align': 'col_align', 'preamble': 'preamble', 'header_start': 'header_start',
-                 'header_end': 'header_end', 'data_start': 'data_start',
-                 'data_end': 'data_end', 'tablefoot': 'tablefoot', 'units': {'col1': 'unit of col1', 'col2': 'unit of col2'}}
-            }
+                          'col_align': 'col_align', 'preamble': 'preamble',
+                          'header_start': 'header_start',
+                          'header_end': 'header_end', 'data_start': 'data_start',
+                          'data_end': 'data_end', 'tablefoot': 'tablefoot',
+                          'units': {'col1': 'unit of col1', 'col2': 'unit of col2'}}
+             }
 
 def add_dictval_to_list(adict, key, alist):
     '''add a value from a dictionary to a list
-   
+
     :param adict: dictionary
     :param key: key of value
     :param list: list where value should be added
@@ -60,7 +63,7 @@ def add_dictval_to_list(adict, key, alist):
 
 def find_latex_line(lines, latex):
     '''Find the first line which matches a patters
-    
+
     :param lines: list of strings
     :param latex: search pattern
     :returns: line number or None, if no match was found
@@ -75,14 +78,14 @@ def find_latex_line(lines, latex):
 
 class LatexHeader(core.BaseHeader):
     header_start = r'\begin{tabular}'
-    
+
     def start_line(self, lines):
         line = find_latex_line(lines, self.header_start)
         if line:
             return line + 1
         else:
             return None
-    
+
     def write(self, lines):
         if not 'col_align' in self.latex.keys():
             self.latex['col_align'] = len(self.cols) * 'c'
@@ -95,21 +98,22 @@ class LatexHeader(core.BaseHeader):
         add_dictval_to_list(self.latex, 'header_start', lines)
         lines.append(self.splitter.join([x.name for x in self.cols]))
         if 'units' in self.latex.keys():
-            lines.append(self.splitter.join([self.latex['units'].get(x.name, ' ') for x in self.cols]))
+            lines.append(self.splitter.join([self.latex['units'].get(x.name, ' ')
+                                             for x in self.cols]))
         add_dictval_to_list(self.latex, 'header_end', lines)
-                
 
-    
+
+
 class LatexData(core.BaseData):
     data_start = None
     data_end = r'\end{tabular}'
-    
+
     def start_line(self, lines):
         if self.data_start:
             return find_latex_line(lines, self.data_start)
         else:
             return self.header.start_line(lines) + 1
-    
+
     def end_line(self, lines):
         if self.data_end:
             return find_latex_line(lines, self.data_end)
@@ -141,127 +145,130 @@ class LatexSplitter(core.BaseSplitter):
         else:
             raise core.InconsistentTableError(r'Lines in LaTeX table have to end with \\')
         return line
-    
+
     def process_val(self, val):
         """Remove whitespace and {} at the beginning or end of value."""
         val = val.strip()
         if val and (val[0] == '{') and (val[-1] == '}'):
             val = val[1:-1]
         return val
-    
+
     def join(self, vals):
         '''Join values together and add a few extra spaces for readability'''
         delimiter = ' ' + self.delimiter + ' '
-        return delimiter.join(str(x) for x in vals) + r' \\'
+        return delimiter.join(x.strip() for x in vals) + r' \\'
 
 class Latex(core.BaseReader):
-    '''Write and read LaTeX tables.
-    
-    This class implements some LaTeX specific commands.
-    Its main purpose is to write out a table in a form that LaTeX
-    can compile. It is beyond the scope of this class to implement every possible 
-    LaTeX command, instead the focus is to generate a syntactically valid LaTeX
-    tables.
-    This class can also read simple LaTeX tables (one line per table row,
-    no ``\multicolumn`` or similar constructs), specifically, it can read the 
-    tables that it writes.
-    
+    r'''Write and read LaTeX tables.
+
+    This class implements some LaTeX specific commands.  Its main
+    purpose is to write out a table in a form that LaTeX can compile. It
+    is beyond the scope of this class to implement every possible LaTeX
+    command, instead the focus is to generate a syntactically valid
+    LaTeX tables.
+
+    This class can also read simple LaTeX tables (one line per table
+    row, no ``\multicolumn`` or similar constructs), specifically, it
+    can read the tables that it writes.
+
     Reading a LaTeX table, the following keywords are accepted:
-    
-    **ignore_latex_commands** : 
+
+    **ignore_latex_commands** :
         Lines starting with these LaTeX commands will be treated as comments (i.e. ignored).
-            
-    When writing a LaTeX table, the some keywords can customize the format.
-    Care has to be taken here, because python interprets ``\\`` in a string as an escape character.
-    In order to pass this to the output either format your strings as raw strings with the ``r`` specifier
-    or use a double ``\\\\``.
+
+    When writing a LaTeX table, the some keywords can customize the
+    format.  Care has to be taken here, because python interprets ``\\``
+    in a string as an escape character.  In order to pass this to the
+    output either format your strings as raw strings with the ``r``
+    specifier or use a double ``\\\\``.
+
     Examples::
-    
+
         caption = r'My table \label{mytable}'
         caption = 'My table \\\\label{mytable}'
-    
+
     **latexdict** : Dictionary of extra parameters for the LaTeX output
-        * tabletype : used for first and last line of table. 
-            The default is ``\\begin{table}``.
-            The following would generate a table, which spans the whole page in a two-column document::
-                
-                asciitable.write(data, sys.stdout, Writer = asciitable.Latex,
-                             latexdict = {'tabletype': 'table*'})
+
+        * tabletype : used for first and last line of table.
+            The default is ``\\begin{table}``.  The following would generate a table,
+            which spans the whole page in a two-column document::
+
+                ascii.write(data, sys.stdout, Writer = ascii.Latex,
+                            latexdict = {'tabletype': 'table*'})
 
         * col_align : Alignment of columns
             If not present all columns will be centered.
 
         * caption : Table caption (string or list of strings)
-            This will appear above the table as it is the standard in many scientific publications.
-            If you prefer a caption below the table, just write the full LaTeX command as
+            This will appear above the table as it is the standard in
+            many scientific publications.  If you prefer a caption below
+            the table, just write the full LaTeX command as
             ``latexdict['tablefoot'] = r'\caption{My table}'``
 
         * preamble, header_start, header_end, data_start, data_end, tablefoot: Pure LaTeX
-            Each one can be a string or a list of strings. These strings will be inserted into the table
-            without any further processing. See the examples below.
+            Each one can be a string or a list of strings. These strings
+            will be inserted into the table without any further
+            processing. See the examples below.
 
         * units : dictionary of strings
-            Keys in this dictionary should be names of columns. If present,
-            a line in the LaTeX table directly below the column names is 
-            added, which contains the values of the dictionary. Example::
-            
-              import asciitable
-              import asciitable.latex
-              import sys
+            Keys in this dictionary should be names of columns. If
+            present, a line in the LaTeX table directly below the column
+            names is added, which contains the values of the
+            dictionary. Example::
+
+              from astropy.io import ascii
               data = {'name': ['bike', 'car'], 'mass': [75,1200], 'speed': [10, 130]}
-              asciitable.write(data, sys.stdout, Writer = asciitable.Latex,
+              ascii.write(data, Writer=ascii.Latex,
                                latexdict = {'units': {'mass': 'kg', 'speed': 'km/h'}})
-            
+
             If the column has no entry in the `units` dictionary, it defaults
             to `' '`.
-                
-        Run the following code to see where each element of the dictionary is inserted in the
-        LaTeX table::
-        
-            import asciitable
-            import asciitable.latex
-            import sys
+
+        Run the following code to see where each element of the
+        dictionary is inserted in the LaTeX table::
+
+            from astropy.io import ascii
             data = {'cola': [1,2], 'colb': [3,4]}
-            asciitable.write(data, sys.stdout, Writer = asciitable.Latex,
-                             latexdict = asciitable.latex.latexdicts['template'])
-                             
-        Some table styles are predefined in the dictionary ``asciitable.latex.latexdicts``. The following generates
-        in table in style preferred by A&A and some other journals::
-        
-            asciitable.write(data, sys.stdout, Writer = asciitable.Latex,
-                             latexdict = asciitable.latex.latexdicts['AA'])
-                             
-        As an example, this generates a table, which spans all columns and is centered on the page::
-        
-            asciitable.write(data, sys.stdout, Writer = asciitable.Latex,
-                             col_align = '|lr|',
-                             latexdict = {'preamble': r'\begin{center}', 'tablefoot': r'\end{center}',
-                                          'tabletype': 'table*'})
-    
-    **caption** : Set table caption 
+            ascii.write(data, Writer=ascii.Latex, latexdict=ascii.latex.latexdicts['template'])
+
+        Some table styles are predefined in the dictionary
+        ``ascii.latex.latexdicts``. The following generates in table in
+        style preferred by A&A and some other journals::
+
+            ascii.write(data, Writer=ascii.Latex, latexdict=ascii.latex.latexdicts['AA'])
+
+        As an example, this generates a table, which spans all columns
+        and is centered on the page::
+
+            ascii.write(data, Writer=ascii.Latex, col_align='|lr|',
+                        latexdict={'preamble': r'\begin{center}',
+                                   'tablefoot': r'\end{center}',
+                                   'tabletype': 'table*'})
+
+    **caption** : Set table caption
         Shorthand for::
-    
+
             latexdict['caption'] = caption
-    
-    **col_align** : Set the column alignment. 
-        If not present this will be auto-generated for centered columns. Shorthand for::
-        
+
+    **col_align** : Set the column alignment.
+        If not present this will be auto-generated for centered
+        columns. Shorthand for::
+
             latexdict['col_align'] = col_align
-    
+
     '''
-    
+
     def __init__(self, ignore_latex_commands = ['hline', 'vspace', 'tableline'], latexdict = {}, caption ='', col_align = None):
 
         core.BaseReader.__init__(self)
         self.header = LatexHeader()
         self.data = LatexData()
-        
+
         self.header.splitter = LatexSplitter()
         self.data.splitter = LatexSplitter()
         self.data.header = self.header
         self.header.data = self.data
         self.latex = {}
-        self.latex['tabletype'] = 'table'
         # The latex dict drives the format of the table and needs to be shared
         # with data and header
         self.header.latex = self.latex
@@ -270,7 +277,7 @@ class Latex(core.BaseReader):
         self.latex.update(latexdict)
         if caption: self.latex['caption'] = caption
         if col_align: self.latex['col_align'] = col_align
-        
+
         self.ignore_latex_commands = ignore_latex_commands
         self.header.comment = '%|' + '|'.join([r'\\' + command for command in self.ignore_latex_commands])
         self.data.comment = self.header.comment
@@ -284,7 +291,7 @@ class Latex(core.BaseReader):
 
 class AASTexHeader(LatexHeader):
     '''In a `deluxetable` some header keywords differ from standard LaTeX.
-    
+
     This header is modified to take that into account.
     '''
     header_start = r'\tablehead'
@@ -324,9 +331,9 @@ class AASTexData(LatexData):
 
 class AASTexHeaderSplitter(LatexSplitter):
     '''extract column names from a `deluxetable`
-    
+
     This splitter expects the following LaTeX code **in a single line**:
-    
+
         \tablehead{\colhead{col1} & ... & \colhead{coln}}
     '''
     def process_line(self, line):
@@ -348,14 +355,14 @@ class AASTexHeaderSplitter(LatexSplitter):
 
 class AASTex(Latex):
     '''Write and read AASTeX tables.
-    
+
     This class implements some AASTeX specific commands.
     AASTeX is used for the AAS (American Astronomical Society)
     publications like ApJ, ApJL and AJ.
-    
-    It derives from :class:`~asciitable.Latex` and accepts the same keywords (see :class:`~asciitable.Latex` for documentation).
-    However, the keywords ``header_start``, ``header_end``, ``data_start`` and ``data_end`` in 
-    ``latexdict`` have no effect.
+
+    It derives from the ``Latex`` reader and accepts the same
+    keywords.  However, the keywords ``header_start``, ``header_end``,
+    ``data_start`` and ``data_end`` in ``latexdict`` have no effect.
     '''
 
     def __init__(self, **kwargs):
@@ -368,8 +375,13 @@ class AASTex(Latex):
         self.data.comment = self.header.comment
         self.data.header = self.header
         self.header.data = self.data
-        self.latex['tabletype'] = 'deluxetable'
         # The latex dict drives the format of the table and needs to be shared
         # with data and header
         self.header.latex = self.latex
         self.data.latex = self.latex
+        # check if tabletype was explicitly set by the user
+        if not (('latexdict' in kwargs) and ('tabletype' in kwargs['latexdict'])):
+            self.latex['tabletype'] = 'deluxetable'
+        self.header.comment = '%|' + '|'.join(
+            [r'\\' + command for command in self.ignore_latex_commands])
+        self.data.comment = self.header.comment
