@@ -1,17 +1,21 @@
+# Licensed under a 3-clause BSD style license - see LICENSE.rst
 import re
-import glob
-import math
 
 import numpy as np
+
+from ....tests.helper import pytest
 from ... import ascii as asciitable
 
-from .common import (raises,
+from .common import (raises, numpy_lt_1p5,
                      assert_equal, assert_almost_equal, assert_true,
                      setup_function, teardown_function, has_isnan)
 
 
 def test_read_all_files():
     for testfile in get_testfiles():
+        if testfile.get('skip'):
+            print('\n\n******** SKIPPING %s' % testfile['name'])
+            continue
         print('\n\n******** READING %s' % testfile['name'])
         for guess in (True, False):
             test_opts = testfile['opts'].copy()
@@ -25,6 +29,9 @@ def test_read_all_files():
 
 def test_guess_all_files():
     for testfile in get_testfiles():
+        if testfile.get('skip'):
+            print('\n\n******** SKIPPING %s' % testfile['name'])
+            continue
         if not testfile['opts'].get('guess', True):
             continue
         print('\n\n******** READING %s' % testfile['name'])
@@ -38,23 +45,42 @@ def test_guess_all_files():
                 assert_equal(len(table[colname]), testfile['nrows'])
 
 
+@pytest.mark.xfail('numpy_lt_1p5')
+def test_daophot_indef():
+    """Test that INDEF is correctly interpreted as a missing value"""
+    table = asciitable.read('t/daophot2.dat', Reader=asciitable.Daophot)
+    for colname in table.colnames:
+        # Three columns have all INDEF values and are masked
+        mask_value = colname in ('OTIME', 'MAG', 'MERR', 'XAIRMASS')
+        assert np.all(table[colname].mask == mask_value)
+
+
+@pytest.mark.xfail('numpy_lt_1p5')
+def test_daophot_types():
+    """
+    Test specific data types which are different from what would be
+    inferred automatically based only data values.  DAOphot reader uses
+    the header information to assign types.
+    """
+    table = asciitable.read('t/daophot2.dat', Reader=asciitable.Daophot)
+    assert table['LID'].dtype.char in 'fd'  # float or double
+    assert table['MAG'].dtype.char in 'fd'  # even without any data values
+    assert table['PIER'].dtype.char in 'US'  # string (data values are consistent with int)
+    assert table['ID'].dtype.char in 'il'  # int or long
+
+
 def test_daophot_header_keywords():
-    reader = asciitable.get_reader(Reader=asciitable.Daophot)
-    table = reader.read('t/daophot.dat')
+    table = asciitable.read('t/daophot.dat', Reader=asciitable.Daophot)
     expected_keywords = (('NSTARFILE', 'test.nst.1', 'filename', '%-23s'),
-                         ('REJFILE', 'hello world', 'filename', '%-23s'),
+                         ('REJFILE', '"hello world"', 'filename', '%-23s'),
                          ('SCALE', '1.',  'units/pix', '%-23.7g'),)
 
+    keywords = table.meta['keywords']  # Ordered dict of keyword structures
     for name, value, units, format_ in expected_keywords:
-        for keyword in reader.keywords:
-            if keyword.name == name:
-                assert_equal(keyword.value, value)
-                assert_equal(keyword.units, units)
-                assert_equal(keyword.format, format_)
-                break
-        else:
-            raise ValueError('Keyword not found')
-
+        keyword = keywords[name]
+        assert_equal(keyword['value'], value)
+        assert_equal(keyword['units'], units)
+        assert_equal(keyword['format'], format_)
 
 
 @raises(asciitable.InconsistentTableError)
@@ -85,8 +111,6 @@ def test_missing_file():
 
 def test_set_names():
     names = ('c1','c2','c3', 'c4', 'c5', 'c6')
-    include_names = ('c1', 'c3')
-    exclude_names = ('c4', 'c5', 'c6')
     data = asciitable.read('t/simple3.txt', names=names, delimiter='|')
     assert_equal(data.dtype.names, names)
 
@@ -105,6 +129,18 @@ def test_set_exclude_names():
     assert_equal(data.dtype.names, ('obsid', 'redshift', 'X', 'rad'))
 
 
+def test_include_names_daophot():
+    include_names = ('ID', 'MAG', 'PIER')
+    data = asciitable.read('t/daophot.dat', include_names=include_names)
+    assert_equal(data.dtype.names, include_names)
+
+
+def test_exclude_names_daophot():
+    exclude_names = ('ID', 'YCENTER', 'MERR', 'NITER', 'CHI', 'PERROR')
+    data = asciitable.read('t/daophot.dat', exclude_names=exclude_names)
+    assert_equal(data.dtype.names, ('XCENTER', 'MAG', 'MSKY', 'SHARPNESS', 'PIER'))
+
+
 def test_custom_process_lines():
     def process_lines(lines):
         bars_at_ends = re.compile(r'^\| | \|$', re.VERBOSE)
@@ -117,6 +153,7 @@ def test_custom_process_lines():
     assert_equal(len(data), 3)
 
 
+@pytest.mark.xfail('numpy_lt_1p5')
 def test_custom_process_line():
     def process_line(line):
         line_out = re.sub(r'^\|\s*', '', line.strip())
@@ -143,7 +180,7 @@ def test_custom_splitters():
     assert_almost_equal(data.field('p1.ampl')[2], 0.000696444029148)
     assert_equal(data.field('statname')[2], 'chi2modvar')
     assert_almost_equal(data.field('statval')[2], 497.56468441)
-    
+
 
 def test_start_end():
     data = asciitable.read('t/test5.dat', header_start=1, data_start=3, data_end=-5)
@@ -164,7 +201,8 @@ def test_set_converters():
 
 def test_from_string():
     f = 't/simple.txt'
-    table = open(f).read()
+    with open(f) as fd:
+        table = fd.read()
     testfile = get_testfiles(f)
     data = asciitable.read(table, **testfile['opts'])
     assert_equal(data.dtype.names, testfile['cols'])
@@ -173,16 +211,17 @@ def test_from_string():
 
 def test_from_filelike():
     f = 't/simple.txt'
-    table = open(f)
     testfile = get_testfiles(f)
-    data = asciitable.read(table, **testfile['opts'])
+    with open(f, 'rb') as fd:
+        data = asciitable.read(fd, **testfile['opts'])
     assert_equal(data.dtype.names, testfile['cols'])
     assert_equal(len(data), testfile['nrows'])
 
 
 def test_from_lines():
     f = 't/simple.txt'
-    table = open(f).readlines()
+    with open(f) as fd:
+        table = fd.readlines()
     testfile = get_testfiles(f)
     data = asciitable.read(table, **testfile['opts'])
     assert_equal(data.dtype.names, testfile['cols'])
@@ -195,16 +234,18 @@ def test_comment_lines():
     assert_equal(table.comment_lines, ['# first comment', '  # second comment'])
 
 
+@pytest.mark.xfail('numpy_lt_1p5')
 def test_fill_values():
     f = 't/fill_values.txt'
     testfile = get_testfiles(f)
     data = asciitable.read(f, fill_values=('a','1'), **testfile['opts'])
-    assert_true((data.mask['a']==[False,True]).all())
-    assert_true((data.data['a']==[1,1]).all())
-    assert_true((data.mask['b']==[False,True]).all())
-    assert_true((data.data['b']==[2,1]).all())
+    assert_true((data['a'].mask==[False,True]).all())
+    assert_true((data['a']==[1,1]).all())
+    assert_true((data['b'].mask==[False,True]).all())
+    assert_true((data['b']==[2,1]).all())
 
 
+@pytest.mark.xfail('numpy_lt_1p5')
 def test_fill_values_col():
     f = 't/fill_values.txt'
     testfile = get_testfiles(f)
@@ -212,6 +253,7 @@ def test_fill_values_col():
     check_fill_values(data)
 
 
+@pytest.mark.xfail('numpy_lt_1p5')
 def test_fill_values_include_names():
     f = 't/fill_values.txt'
     testfile = get_testfiles(f)
@@ -220,6 +262,7 @@ def test_fill_values_include_names():
     check_fill_values(data)
 
 
+@pytest.mark.xfail('numpy_lt_1p5')
 def test_fill_values_exclude_names():
     f = 't/fill_values.txt'
     testfile = get_testfiles(f)
@@ -228,31 +271,50 @@ def test_fill_values_exclude_names():
     check_fill_values(data)
 
 
+@pytest.mark.xfail('numpy_lt_1p5')
 def check_fill_values(data):
     """compare array column by column with expectation """
-    assert_true((data.mask['a']==[False,False]).all())
-    assert_true((data.data['a']==['1','a']).all())
-    assert_true((data.mask['b']==[False,True]).all())
-    assert_true((data.data['b']==[2,1]).all())        
+    assert_true((data['a'].mask==[False,False]).all())
+    assert_true((data['a']==['1','a']).all())
+    assert_true((data['b'].mask==[False,True]).all())
+    # Check that masked value is "do not care" in comparison
+    assert_true((data['b']==[2, -999]).all())
+    data['b'].mask = False  # explicitly unmask for comparison
+    assert_true((data['b']==[2,1]).all())
 
 
+@pytest.mark.xfail('numpy_lt_1p5')
 def test_fill_values_list():
     f = 't/fill_values.txt'
     testfile = get_testfiles(f)
     data = asciitable.read(f, fill_values=[('a','42'),('1','42','a')],
                            **testfile['opts'])
-    assert_true((data.data['a']==[42,42]).all())
+    data['a'].mask = False  # explicitly unmask for comparison
+    assert_true((data['a']==[42,42]).all())
 
 
+@pytest.mark.xfail('numpy_lt_1p5')
 def test_masking_Cds():
     f = 't/cds.dat'
     testfile = get_testfiles(f)
-    data = asciitable.read(f, 
+    data = asciitable.read(f,
                            **testfile['opts'])
     assert_true(data['AK'].mask[0])
     assert_true(not data['Fit'].mask[0])
 
 
+@pytest.mark.xfail('numpy_lt_1p5')
+def test_null_Ipac():
+    f = 't/ipac.dat'
+    testfile = get_testfiles(f)
+    data = asciitable.read(f, **testfile['opts'])
+    mask = np.array([(True, False, True, False, True),
+                     (False, False, False, False, False)],
+                    dtype=[('ra', '|b1'), ('dec', '|b1'), ('sai', '|b1'),
+                           ('v2', '|b1'), ('sptype', '|b1')])
+    assert np.all(data.mask == mask)
+
+
 def test_set_guess_kwarg():
     """Read a file using guess with one of the typical guess_kwargs explicitly set."""
     data = asciitable.read('t/space_delim_no_header.dat',
@@ -296,12 +358,16 @@ def get_testfiles(name=None):
                   'Fit'),
          'name': 't/cds.dat',
          'nrows': 1,
-         'opts': {'Reader': asciitable.Cds,
-                  'Outputter': asciitable.NumpyOutputter}},
+         'skip': numpy_lt_1p5,
+         'opts': {'Reader': asciitable.Cds}},
         {'cols': ('a', 'b', 'c'),
          'name': 't/commented_header.dat',
          'nrows': 2,
          'opts': {'Reader': asciitable.CommentedHeader}},
+        {'cols': ('a', 'b', 'c'),
+         'name': 't/commented_header2.dat',
+         'nrows': 2,
+         'opts': {'Reader': asciitable.CommentedHeader, 'header_start': -1}},
         {'cols': ('col1', 'col2', 'col3', 'col4', 'col5'),
          'name': 't/continuation.dat',
          'nrows': 2,
@@ -322,9 +388,20 @@ def get_testfiles(name=None):
          'nrows': 2,
          'requires_numpy': True,
          'opts': {'Reader': asciitable.Daophot}},
+        {'cols': ('NUMBER',
+                  'FLUX_ISO',
+                  'FLUXERR_ISO',
+                  'VALUES',
+                  'VALUES_1',
+                  'FLAG'),
+         'name': 't/sextractor.dat',
+         'nrows': 3,
+         'requires_numpy': True,
+         'opts': {'Reader': asciitable.SExtractor}},
         {'cols': ('ra', 'dec', 'sai', 'v2', 'sptype'),
          'name': 't/ipac.dat',
          'nrows': 2,
+         'skip': numpy_lt_1p5,
          'opts': {'Reader': asciitable.Ipac}},
         {'cols': ('col0',
                   'objID',
@@ -346,6 +423,7 @@ def get_testfiles(name=None):
                   'detlim90',
                   'fBlim90'),
          'name': 't/nls1_stackinfo.dbout',
+         'skip': numpy_lt_1p5,
          'nrows': 58,
          'opts': {'data_start': 2, 'delimiter': '|', 'guess': False}},
         {'cols': ('Index',
@@ -361,6 +439,7 @@ def get_testfiles(name=None):
                   'AK',
                   'Fit'),
          'name': 't/no_data_cds.dat',
+         'skip': numpy_lt_1p5,
          'nrows': 0,
          'opts': {'Reader': asciitable.Cds}},
         {'cols': ('ID',
@@ -378,10 +457,26 @@ def get_testfiles(name=None):
          'nrows': 0,
          'requires_numpy': True,
          'opts': {'Reader': asciitable.Daophot}},
+        {'cols': ('NUMBER',
+                  'FLUX_ISO',
+                  'FLUXERR_ISO',
+                  'VALUES',
+                  'VALUES_1',
+                  'FLAG'),
+         'name': 't/no_data_sextractor.dat',
+         'nrows': 0,
+         'requires_numpy': True,
+         'opts': {'Reader': asciitable.SExtractor}},
         {'cols': ('ra', 'dec', 'sai', 'v2', 'sptype'),
          'name': 't/no_data_ipac.dat',
          'nrows': 0,
+         'skip': numpy_lt_1p5,
          'opts': {'Reader': asciitable.Ipac}},
+        {'cols': ('ra', 'v2'),
+         'name': 't/ipac.dat',
+         'nrows': 2,
+         'skip': numpy_lt_1p5,
+         'opts': {'Reader': asciitable.Ipac, 'include_names': ['ra', 'v2']}},
         {'cols': ('a', 'b', 'c'),
          'name': 't/no_data_with_header.dat',
          'nrows': 0,
