@@ -544,9 +544,10 @@ def info(filename, output=None, **kwargs):
         FITS file to obtain info from.  If opened, mode must be one of
         the following: rb, rb+, or ab+.
 
-    output : file (optional)
-        File-like object to output the HDU info to.  Outputs to stdout by
-        default.
+    output : file, bool (optional)
+        A file-like object to write the output to.  If ``False``, does not
+        output to a file and instead returns a list of tuples representing the
+        HDU info.  Writes to ``sys.stdout`` by default.
 
     kwargs
         Any additional keyword arguments to be passed to
@@ -758,7 +759,7 @@ def _makehdu(data, header):
         # The HDU type was unrecognized, possibly due to a
         # nonexistent/incomplete header
         if ((isinstance(data, np.ndarray) and data.dtype.fields is not None)
-            or isinstance(data, np.recarray)):
+                or isinstance(data, np.recarray)):
             hdu = BinTableHDU(data)
         elif isinstance(data, np.ndarray):
             hdu = ImageHDU(data)
