@@ -104,6 +104,12 @@ class _BaseHDU(object):
         self._hdrLoc = None
         self._datLoc = None
         self._datSpan = None
+
+        # This internal variable is used to track whether the data attribute
+        # still points to the same data array as when the HDU was originally
+        # created (this does not track whether the data is actually the same
+        # content-wise)
+        self._data_replaced = False
         self._new = True
         self._output_checksum = False
 
@@ -206,7 +212,7 @@ class _BaseHDU(object):
         if isinstance(data, Header):
             header = data
             if (not len(header) or
-                header.keys()[0] not in ('SIMPLE', 'XTENSION')):
+                    header.keys()[0] not in ('SIMPLE', 'XTENSION')):
                 raise ValueError('Block does not begin with SIMPLE or '
                                  'XTENSION')
         else:
@@ -410,7 +416,7 @@ class _BaseHDU(object):
         # If data is unsigned integer 16, 32 or 64, remove the
         # BSCALE/BZERO cards
         if (self._data_loaded and self.data is not None and
-            self._standard and _is_pseudo_unsigned(self.data.dtype)):
+                self._standard and _is_pseudo_unsigned(self.data.dtype)):
             for keyword in ('BSCALE', 'BZERO'):
                 try:
                     del self._header[keyword]
@@ -528,7 +534,7 @@ class _BaseHDU(object):
                 # Seek through the array's bases for an memmap'd array; we
                 # can't rely on the _File object to give us this info since the
                 # user may have replaced the previous mmap'd array
-                if copy:
+                if copy or self._data_replaced:
                     # Of course, if we're copying the data to a new file we
                     # don't care about flushing the original mmap; instead just
                     # read it into the new file
@@ -549,6 +555,7 @@ class _BaseHDU(object):
         self._hdrLoc = hdrloc
         self._datLoc = datloc
         self._datSpan = datsize + _pad_length(datsize)
+        self._data_replaced = False
 
 _AllHDU = _BaseHDU  # For backwards-compatibility, though nobody should have
                     # been using this directly
@@ -962,7 +969,6 @@ class _ValidHDU(_BaseHDU, _Verify):
                             self.run_option(option=option, err_text=err_text,
                                             fix=fix, fix_text="Deleted."))
 
-
         # Verify that the EXTNAME keyword exists and is a string
         if 'EXTNAME' in self._header:
             if not isinstance(self._header['EXTNAME'], basestring):
