@@ -75,7 +75,7 @@ class CompImageHeader(Header):
                             'ZCMPTYPE') and
             keyword[:4] not in ('ZVAL') and
             keyword[:5] not in ('NAXIS', 'TTYPE', 'TFORM', 'ZTILE', 'ZNAME')
-            and keyword[:6] not in ('ZNAXIS')):
+                and keyword[:6] not in ('ZNAXIS')):
             self._table_header.set(keyword, value, comment, before, after)
 
     def add_history(self, value, before=None, after=None):
@@ -154,9 +154,8 @@ class CompImageHDU(BinTableHDU):
             2) The `CompImageHDU` class supports the FITS tiled image
                compression convention in which the image is subdivided into a
                grid of rectangular tiles, and each tile of pixels is
-               individually compressed.
-               The details of this FITS compression convention are described at
-               the `FITS Support Office web site
+               individually compressed.  The details of this FITS compression
+               convention are described at the `FITS Support Office web site
                <http://fits.gsfc.nasa.gov/registry/tilecompression.html>`_.
                Basically, the compressed image tiles are stored in rows of a
                variable length arrray column in a FITS binary table.  The
@@ -414,7 +413,8 @@ class CompImageHDU(BinTableHDU):
             self._header.set('ZCMPTYPE', compressionType,
                              'compression algorithm', after='TFIELDS')
         else:
-            compressionType = self._header.get('ZCMPTYPE', 'RICE_1')
+            compressionType = self._header.get('ZCMPTYPE',
+                                                DEFAULT_COMPRESSION_TYPE)
 
         # If the input image header had BSCALE/BZERO cards, then insert
         # them in the table header.
@@ -1154,7 +1154,7 @@ class CompImageHDU(BinTableHDU):
         # is the default value of COMPRESSED_IMAGE.
 
         if ('EXTNAME' in self._header and
-            self._header['EXTNAME'] == 'COMPRESSED_IMAGE'):
+                self._header['EXTNAME'] == 'COMPRESSED_IMAGE'):
             del self._image_header['EXTNAME']
 
         # Look to see if there are any blank cards in the table
@@ -1210,19 +1210,12 @@ class CompImageHDU(BinTableHDU):
         image_bitpix = _ImageBaseHDU.ImgCode[self.data.dtype.name]
 
         if (self.header.get('NAXIS', 0) != len(self.data.shape) or
-            self.header.get('BITPIX', 0) != image_bitpix or
-            self._header.get('ZNAXIS', 0) != len(self.data.shape) or
-            self._header.get('ZBITPIX', 0) != image_bitpix or
-            self.shape != self.data.shape):
+                self.header.get('BITPIX', 0) != image_bitpix or
+                self._header.get('ZNAXIS', 0) != len(self.data.shape) or
+                self._header.get('ZBITPIX', 0) != image_bitpix or
+                self.shape != self.data.shape):
             self.updateHeaderData(self.header)
 
-        # put data in machine native byteorder on little endian machines
-        # for handing off to the compression code
-        if sys.byteorder == 'little':
-            swap_types = ('>',)
-        else:
-            swap_types = ('>', '=')
-
         # TODO: This is copied right out of _ImageBaseHDU._writedata_internal;
         # it would be cool if we could use an internal ImageHDU and use that to
         # write to a buffer for compression or something. See ticket #88
@@ -1232,24 +1225,23 @@ class CompImageHDU(BinTableHDU):
             # Convert the unsigned array to signed
             self.data = np.array(
                 self.data - _unsigned_zero(self.data.dtype),
-                dtype='<i%d' % self.data.dtype.itemsize)
+                dtype='=i%d' % self.data.dtype.itemsize)
             should_swap = False
         else:
-            byteorder = self.data.dtype.str[0]
-            should_swap = (byteorder in swap_types)
+            should_swap = not self.data.dtype.isnative
 
         if should_swap:
             self.data.byteswap(True)
 
-        nrows = self._header['NAXIS2']
-        tbsize = self._header['NAXIS1'] * nrows
-
-        self._header['PCOUNT'] = 0
-        if 'THEAP' in self._header:
-            del self._header['THEAP']
-        self._theap = tbsize
-
         try:
+            nrows = self._header['NAXIS2']
+            tbsize = self._header['NAXIS1'] * nrows
+
+            self._header['PCOUNT'] = 0
+            if 'THEAP' in self._header:
+                del self._header['THEAP']
+            self._theap = tbsize
+
             # Compress the data.
             # The current implementation of compress_hdu assumes the empty
             # compressed data table has already been initialized in
@@ -1263,18 +1255,6 @@ class CompImageHDU(BinTableHDU):
                 self.data.byteswap(True)
             self.data = old_data
 
-        # Chances are not all the space allocated for the compressed data was
-        # needed.  If not, go ahead and truncate the array:
-        dataspan = tbsize + heapsize
-        if len(self.compData) > dataspan:
-            if self.compData.flags.owndata:
-                self.compData.resize(dataspan)
-            else:
-                # Need to copy to a new array; this generally shouldn't happen
-                # at all though there are some contrived cases (such as in one
-                # of the regression tests) where it can happen.
-                self.compData = np.resize(self.compData, (dataspan,))
-
         dtype = np.rec.format_parser(','.join(self.columns._recformats),
                                      self.columns.names, None).dtype
         # CFITSIO will write the compressed data in big-endian order
@@ -1521,7 +1501,7 @@ class CompImageHDU(BinTableHDU):
 
     def _update_header_scale_info(self, dtype=None):
         if (not self._do_not_scale_image_data and
-            not (self._orig_bzero == 0 and self._orig_bscale == 1)):
+                not (self._orig_bzero == 0 and self._orig_bscale == 1)):
             for keyword in ['BSCALE', 'BZERO']:
                 # Make sure to delete from both the image header and the table
                 # header; later this will be streamlined
