@@ -21,12 +21,6 @@ class _ImageBaseHDU(_ValidHDU):
 
     data
         image data
-
-    _file
-        file associated with array
-
-    _datLoc
-        starting byte location of data block in file
     """
 
     # mappings between FITS and numpy typecodes
@@ -175,7 +169,6 @@ class _ImageBaseHDU(_ValidHDU):
 
         return Section(self)
 
-
     @property
     def shape(self):
         """
@@ -207,9 +200,15 @@ class _ImageBaseHDU(_ValidHDU):
 
     @data.setter
     def data(self, data):
-        self.__dict__['data'] = data
-        self._modified = True
-        if self.data is not None and not isinstance(data, np.ndarray):
+        if 'data' in self.__dict__:
+            if self.__dict__['data'] is data:
+                return
+            else:
+                self._data_replaced = True
+        else:
+            self._data_replaced = True
+
+        if data is not None and not isinstance(data, np.ndarray):
             # Try to coerce the data into a numpy array--this will work, on
             # some level, for most objects
             try:
@@ -218,6 +217,9 @@ class _ImageBaseHDU(_ValidHDU):
                 raise TypeError('data object %r could not be coerced into an '
                                 'ndarray' % data)
 
+        self.__dict__['data'] = data
+        self._modified = True
+
         if isinstance(data, np.ndarray):
             self._bitpix = _ImageBaseHDU.ImgCode[data.dtype.name]
             self._orig_bitpix = self._bitpix
@@ -232,6 +234,10 @@ class _ImageBaseHDU(_ValidHDU):
 
         self.update_header()
 
+        # returning the data signals to lazyproperty that we've already handled
+        # setting self.__dict__['data']
+        return data
+
     def update_header(self):
         """
         Update the header keywords to agree with the data.
@@ -295,7 +301,7 @@ class _ImageBaseHDU(_ValidHDU):
 
     def _update_header_scale_info(self, dtype=None):
         if (not self._do_not_scale_image_data and
-            not (self._orig_bzero == 0 and self._orig_bscale == 1)):
+                not (self._orig_bzero == 0 and self._orig_bscale == 1)):
             for keyword in ['BSCALE', 'BZERO']:
                 try:
                     del self._header[keyword]
@@ -524,7 +530,7 @@ class _ImageBaseHDU(_ValidHDU):
         raw_data.dtype = raw_data.dtype.newbyteorder('>')
 
         if (self._orig_bzero == 0 and self._orig_bscale == 1 and
-            self._blank is None):
+                self._blank is None):
             # No further conversion of the data is necessary
             return raw_data
 
@@ -632,7 +638,7 @@ class _ImageBaseHDU(_ValidHDU):
             # base class.  The other possibility is that there is no data at
             # all.  This can also be handled in a gereric manner.
             return super(_ImageBaseHDU, self)._calculate_datasum(
-                    blocking=blocking)
+                blocking=blocking)
 
 
 class Section(object):
