@@ -20,7 +20,7 @@ if PY3:  # pragma: py3
         if isinstance(s, str):
             return s.encode('ascii')
         elif isinstance(s, numpy.ndarray) and \
-             issubclass(s.dtype.type, numpy.str_):
+                issubclass(s.dtype.type, numpy.str_):
             ns = numpy.char.encode(s, 'ascii').view(type(s))
             if ns.dtype.itemsize != s.dtype.itemsize / 4:
                 ns = ns.astype((numpy.bytes_, s.dtype.itemsize / 4))
@@ -37,7 +37,14 @@ if PY3:  # pragma: py3
             # array, hence the view() call
             # It also doesn't necessarily preserve widths of the strings,
             # hence the astype()
-            ns = numpy.char.decode(s, 'ascii').view(type(s))
+            if s.size == 0:
+                # Numpy apparently also has a bug that if a string array is
+                # empty calling np.char.decode on it returns an empty float64
+                # array wth
+                dt = s.dtype.str.replace('S', 'U')
+                ns = numpy.array([], dtype=dt).view(type(s))
+            else:
+                ns = numpy.char.decode(s, 'ascii').view(type(s))
             if ns.dtype.itemsize / 4 != s.dtype.itemsize:
                 ns = ns.astype((numpy.str_, s.dtype.itemsize))
             return ns
@@ -165,12 +172,12 @@ if PY3:  # pragma: py3
 
             if 'order' in _recarray.__new__.__code__.co_varnames:
                 return _recarray.__new__(
-                        subtype, shape, dtype, buf, offset, strides, formats,
-                        names, titles, byteorder, aligned, order)
+                    subtype, shape, dtype, buf, offset, strides, formats,
+                    names, titles, byteorder, aligned, order)
             else:
                 return _recarray.__new__(
-                        subtype, shape, dtype, buf, offset, strides, formats,
-                        names, titles, byteorder, aligned)
+                    subtype, shape, dtype, buf, offset, strides, formats,
+                    names, titles, byteorder, aligned)
     numpy.recarray = numpy.core.records.recarray = recarray
 
     # We also need to patch astropy.io.fits.file._File which can also be
