@@ -510,6 +510,7 @@ void configure_compression(fitsfile* fileptr, PyObject* header) {
     char* zname;
     int znaxis;
     char* tmp;
+    float version;
 
     unsigned int idx;
 
@@ -639,11 +640,32 @@ void configure_compression(fitsfile* fileptr, PyObject* header) {
         } else if (Fptr->zbitpix < 0 && 0 == strcmp(zname, "NOISEBIT")) {
              get_header_float(header, keyword, &(Fptr->quantize_level),
                               DEFAULT_QUANTIZE_LEVEL);
+             if (Fptr->quantize_level == 0.0) {
+                 /* NOISEBIT == 0 is equivalent to no quantize */
+                 Fptr->quantize_level = NO_QUANTIZE;
+             }
         }
 
         idx++;
     }
 
+    /* The ZQUANTIZ keyword determines the quantization algorithm; NO_QUANTIZE
+       implies lossless compression */
+    if (0 == get_header_string(header, "ZQUANTIZ", &tmp, "")) {
+        /* Ugh; the fact that cfitsio defines its version as a float makes
+           preprocessor comparison impossible */
+        fits_get_version(&version);
+        if ((version >= CFITSIO_LOSSLESS_COMP_SUPPORTED_VERS) &&
+                (0 == strcmp(tmp, "NONE"))) {
+            Fptr->quantize_level = NO_QUANTIZE;
+        } else if (0 == strcmp(tmp, "SUBTRACTIVE_DITHER_1")) {
+            Fptr->quantize_dither = SUBTRACTIVE_DITHER_1;
+        } else {
+            Fptr->quantize_dither = 0;
+        }
+    } else {
+        Fptr->quantize_dither = 0;
+    }
 
     Fptr->compressimg = 1;
     Fptr->maxelem = imcomp_calc_max_elem(Fptr->compress_type,
@@ -655,42 +677,90 @@ void configure_compression(fitsfile* fileptr, PyObject* header) {
 }
 
 
-void open_from_hdu(fitsfile** fileptr, void** buf, size_t* bufsize,
-                   PyObject* hdu, tcolumn* columns) {
-    PyObject* header = NULL;
-    PyArrayObject* data = NULL;
-    PyArrayObject* base;
-    PyArrayObject* tmp;
-    FITSfile* Fptr;
+void init_output_buffer(PyObject* hdu, void** buf, size_t* bufsize) {
+    // Determines a good size for the output data buffer and allocates
+    // memory for it, returning the address and size of the allocated
+    // membory in to **buf and *bufsize respectively.
 
-    int status = 0;
+    PyObject* header = NULL;
+    char keyword[9];
+    char* tmp;
+    int znaxis;
+    int idx;
+    int compress_type;
+    int zbitpix;
+    int rice_blocksize = 0;
     long long rowlen;
     long long nrows;
-    long long heapsize;
-    long long theap;
+    long maxelem;
+    long tilelen;
+    unsigned long maxtilelen = 1;
 
     header = PyObject_GetAttrString(hdu, "_header");
     if (header == NULL) {
         goto fail;
     }
 
-    data = (PyArrayObject*) PyObject_GetAttrString(hdu, "compData");
-    if (data == NULL) {
+    if (0 != get_header_int(header, "ZNAXIS", &znaxis, 0)) {
         goto fail;
     }
 
+    for (idx = 1; idx <= znaxis; idx++) {
+        snprintf(keyword, 9, "ZTILE%u", idx);
+        get_header_long(header, keyword, &tilelen, 1);
+        maxtilelen *= tilelen;
+    }
+
+    get_header_string(header, "ZCMPTYPE", &tmp, DEFAULT_COMPRESSION_TYPE);
+    compress_type = compress_type_from_string(tmp);
+    if (compress_type == RICE_1) {
+        get_header_int(header, "ZVAL1", &rice_blocksize, 0);
+    }
 
     get_header_longlong(header, "NAXIS1", &rowlen, 0);
     get_header_longlong(header, "NAXIS2", &nrows, 0);
 
-    // The PCOUNT keyword contains the number of bytes in the table heap
-    get_header_longlong(header, "PCOUNT", &heapsize, 0);
+    // Get the ZBITPIX header value; if this is missing we're in trouble
+    if (0 != get_header_int(header, "ZBITPIX", &zbitpix, 0)) {
+        goto fail;
+    }
 
-    // The THEAP keyword gives the offset of the heap from the beginning of
-    // the HDU data portion; normally this offset is 0 but it can be set
-    // to something else with THEAP
-    get_header_longlong(header, "THEAP", &theap, 0);
+    maxelem = imcomp_calc_max_elem(compress_type, maxtilelen, zbitpix,
+                                   rice_blocksize);
 
+    *bufsize = ((size_t) (rowlen * nrows) + (nrows * maxelem));
+
+    if (*bufsize < IOBUFLEN) {
+        // We must have a full FITS block at a minimum
+        *bufsize = IOBUFLEN;
+    } else if (*bufsize % IOBUFLEN != 0) {
+        // Still make sure to pad out to a multiple of 2880 byte blocks
+        // otherwise CFITSIO can get read errors when it tries to read
+        // a partial block that goes past the end of the file
+        *bufsize += ((size_t) (IOBUFLEN - (*bufsize % IOBUFLEN)));
+    }
+
+    // TODO: Add error handling here
+    *buf = calloc(*bufsize, sizeof(char));
+fail:
+    Py_XDECREF(header);
+    return;
+}
+
+
+void get_hdu_data_base(PyObject* hdu, void** buf, size_t* bufsize) {
+    // Given a pointer to an HDU object, returns a pointer to the deepest base
+    // array of that HDU's data array into **buf, and the size of that array
+    // into *bufsize.
+
+    PyArrayObject* data = NULL;
+    PyArrayObject* base;
+    PyArrayObject* tmp;
+
+    data = (PyArrayObject*) PyObject_GetAttrString(hdu, "compData");
+    if (data == NULL) {
+        goto fail;
+    }
 
     // Walk the array data bases until we find the lowest ndarray base; for
     // CompImageHDUs there should always be at least one contiguous byte array
@@ -712,9 +782,41 @@ void open_from_hdu(fitsfile** fileptr, void** buf, size_t* bufsize,
     }
 
     *buf = PyArray_DATA(base);
+fail:
+    Py_XDECREF(data);
+    return;
+}
+
+
+void open_from_hdu(fitsfile** fileptr, void** buf, size_t* bufsize,
+                   PyObject* hdu, tcolumn** columns) {
+    PyObject* header = NULL;
+    FITSfile* Fptr;
+
+    int status = 0;
+    long long rowlen;
+    long long nrows;
+    long long heapsize;
+    long long theap;
+
+    header = PyObject_GetAttrString(hdu, "_header");
+    if (header == NULL) {
+        goto fail;
+    }
+
+    get_header_longlong(header, "NAXIS1", &rowlen, 0);
+    get_header_longlong(header, "NAXIS2", &nrows, 0);
+
+    // The PCOUNT keyword contains the number of bytes in the table heap
+    get_header_longlong(header, "PCOUNT", &heapsize, 0);
+
+    // The THEAP keyword gives the offset of the heap from the beginning of
+    // the HDU data portion; normally this offset is 0 but it can be set
+    // to something else with THEAP
+    get_header_longlong(header, "THEAP", &theap, 0);
+
+    fits_create_memfile(fileptr, buf, bufsize, 0, realloc, &status);
 
-    fits_create_memfile(fileptr, buf, bufsize, 0, compression_realloc,
-                        &status);
     if (status != 0) {
         process_status_err(status);
         goto fail;
@@ -741,7 +843,7 @@ void open_from_hdu(fitsfile** fileptr, void** buf, size_t* bufsize,
 
     // Configure the array of table column structs from the Astropy header
     // instead of allowing CFITSIO to try to read from the header
-    tcolumns_from_header(*fileptr, header, &columns);
+    tcolumns_from_header(*fileptr, header, columns);
     if (PyErr_Occurred()) {
         goto fail;
     }
@@ -752,7 +854,6 @@ void open_from_hdu(fitsfile** fileptr, void** buf, size_t* bufsize,
 
 fail:
     Py_XDECREF(header);
-    Py_XDECREF(data);
     return;
 }
 
@@ -764,19 +865,18 @@ PyObject* compression_compress_hdu(PyObject* self, PyObject* args)
     PyObject* retval = NULL;
     tcolumn* columns = NULL;
 
-    void* orig_outbuf;
-    size_t orig_outbufsize;
     void* outbuf;
     size_t outbufsize;
 
     PyArrayObject* indata;
     PyArrayObject* tmp;
-    long znaxis;
+    npy_intp znaxis;
     int datatype;
     int npdatatype;
     unsigned long long heapsize;
 
     fitsfile* fileptr;
+    FITSfile* Fptr;
     int status = 0;
 
     if (!PyArg_ParseTuple(args, "O:compression.compress_hdu", &hdu))
@@ -790,14 +890,15 @@ PyObject* compression_compress_hdu(PyObject* self, PyObject* args)
     // too much confusion to Astropy's internal book keeping.
     // We just need to get the compressed bytes and Astropy will handle the
     // writing of them.
-    open_from_hdu(&fileptr, &outbuf, &outbufsize, hdu, columns);
+    init_output_buffer(hdu, &outbuf, &outbufsize);
+    open_from_hdu(&fileptr, &outbuf, &outbufsize, hdu, &columns);
     if (PyErr_Occurred()) {
         return NULL;
     }
-    orig_outbuf = outbuf;
-    orig_outbufsize = outbufsize;
 
-    bitpix_to_datatypes(fileptr->Fptr->zbitpix, &datatype, &npdatatype);
+    Fptr = fileptr->Fptr;
+
+    bitpix_to_datatypes(Fptr->zbitpix, &datatype, &npdatatype);
     if (PyErr_Occurred()) {
         return NULL;
     }
@@ -817,26 +918,34 @@ PyObject* compression_compress_hdu(PyObject* self, PyObject* args)
         goto fail;
     }
 
-    // It's possible that between calls to fits_write_img and fits_flush_buffer
-    // the size of the output buffer was insufficient and the buffer had to be
-    // reallocated.  The address and size of the new buffer should be in
-    // outbuf and outbufsize.  We need to create a new PyArrayObject using the
-    // new buffer and size
-    if (orig_outbuf != outbuf || orig_outbufsize != outbufsize) {
-        // It's possible, albeit unlikely, that realloc can return a block of
-        // memory with the same address but different size.
-        znaxis = (long) outbufsize;  // The output array is just one dimension.
-        tmp = (PyArrayObject*) PyArray_SimpleNewFromData(1, &znaxis, NPY_UBYTE,
-                                                         outbuf);
-        PyObject_SetAttrString(hdu, "compData", (PyObject*) tmp);
-        Py_DECREF(tmp);
+    // Previously this used outbufsize as the size to use for the new Numpy
+    // byte array. However outbufsize is usually larger than necessary to
+    // store all the compressed data exactly; instead use the exact size
+    // of the compressed data from the heapsize plus the size of the table
+    // itself
+    heapsize = (unsigned long long) Fptr->heapsize;
+    znaxis = (npy_intp) (Fptr->heapstart + heapsize);
+
+    if (znaxis < outbufsize) {
+        // Go ahead and truncate to the size in znaxis to free the
+        // redundant allocation
+        // TODO: Add error handling
+        outbuf = realloc(outbuf, (size_t) znaxis);
     }
 
-    heapsize = (unsigned long long) fileptr->Fptr->heapsize;
+    tmp = (PyArrayObject*) PyArray_SimpleNewFromData(1, &znaxis, NPY_UBYTE,
+                                                     outbuf);
 
-    retval = PyLong_FromUnsignedLongLong(heapsize);
+
+    // Leaves refcount of tmp untouched, so its refcount should remain as 1
+    retval = Py_BuildValue("KN", heapsize, tmp);
 
 fail:
+    if (columns != NULL) {
+        PyMem_Free(columns);
+        Fptr->tableptr = NULL;
+    }
+
     if (fileptr != NULL) {
         status = 1; // Disable header-related errors
         fits_close_file(fileptr, &status);
@@ -846,9 +955,6 @@ fail:
         }
     }
 
-    if (columns != NULL) {
-        PyMem_Free(columns);
-    }
     Py_XDECREF(indata);
 
     // Clear any messages remaining in CFITSIO's error stack
@@ -869,8 +975,8 @@ PyObject* compression_decompress_hdu(PyObject* self, PyObject* args)
     PyArrayObject* outdata;
     int datatype;
     int npdatatype;
-    int zndim;
-    long* znaxis;
+    npy_intp zndim;
+    npy_intp* znaxis;
     long arrsize;
     unsigned int idx;
 
@@ -884,7 +990,13 @@ PyObject* compression_decompress_hdu(PyObject* self, PyObject* args)
         return NULL;
     }
 
-    open_from_hdu(&fileptr, &inbuf, &inbufsize, hdu, columns);
+    // Grab a pointer to the input data from the HDU's compData attribute
+    get_hdu_data_base(hdu, &inbuf, &inbufsize);
+    if (PyErr_Occurred()) {
+        return NULL;
+    }
+
+    open_from_hdu(&fileptr, &inbuf, &inbufsize, hdu, &columns);
     if (PyErr_Occurred()) {
         return NULL;
     }
@@ -894,8 +1006,8 @@ PyObject* compression_decompress_hdu(PyObject* self, PyObject* args)
         return NULL;
     }
 
-    zndim = fileptr->Fptr->zndim;
-    znaxis = (long*) PyMem_Malloc(sizeof(long) * zndim);
+    zndim = (npy_intp)fileptr->Fptr->zndim;
+    znaxis = (npy_intp*) PyMem_Malloc(sizeof(npy_intp) * zndim);
     arrsize = 1;
     for (idx = 0; idx < zndim; idx++) {
         znaxis[zndim - idx - 1] = fileptr->Fptr->znaxis[idx];
@@ -914,6 +1026,11 @@ PyObject* compression_decompress_hdu(PyObject* self, PyObject* args)
     }
 
 fail:
+    if (columns != NULL) {
+        PyMem_Free(columns);
+        fileptr->Fptr->tableptr = NULL;
+    }
+
     if (fileptr != NULL) {
         status = 1;// Disable header-related errors
         fits_close_file(fileptr, &status);
@@ -923,9 +1040,6 @@ fail:
         }
     }
 
-    if (columns != NULL) {
-        PyMem_Free(columns);
-    }
     PyMem_Free(znaxis);
 
     // Clear any messages remaining in CFITSIO's error stack
@@ -935,24 +1049,6 @@ fail:
 }
 
 
-PyObject* compression_calc_max_elem(PyObject* self, PyObject* args) {
-    char* zcmptype;
-    long maxtilelen;
-    int zbitpix;
-    int rice_blocksize;
-
-    if (!PyArg_ParseTuple(args, "slii:compression.calc_max_elem", &zcmptype,
-                          &maxtilelen, &zbitpix, &rice_blocksize)) {
-        PyErr_SetString(PyExc_TypeError, "Couldn't parse arguments");
-        return NULL;
-    }
-
-    return PyLong_FromLong(
-        (long) imcomp_calc_max_elem(compress_type_from_string(zcmptype),
-                                    maxtilelen, zbitpix, rice_blocksize));
-}
-
-
 /* CFITSIO version float as returned by fits_get_version() */
 static double cfitsio_version;
 
@@ -984,7 +1080,6 @@ static PyMethodDef compression_methods[] =
 {
    {"compress_hdu", compression_compress_hdu, METH_VARARGS},
    {"decompress_hdu", compression_decompress_hdu, METH_VARARGS},
-   {"calc_max_elem", compression_calc_max_elem, METH_VARARGS},
    {NULL, NULL}
 };
 
