@@ -15,9 +15,9 @@ class TestDivisionFunctions(FitsTestCase):
         t1 = fits.open(self.data('tb.fits'))
         s = t1[1].data.tostring()
         a1 = np.rec.array(
-                s,
-                dtype=np.dtype([('c1', '>i4'), ('c2', '|S3'),
-                                ('c3', '>f4'), ('c4', '|i1')]))
+            s,
+            dtype=np.dtype([('c1', '>i4'), ('c2', '|S3'),
+                         ('c3', '>f4'), ('c4', '|i1')]))
 
     def test_card_with_continue(self):
         h = fits.PrimaryHDU()
@@ -38,5 +38,5 @@ class TestDivisionFunctions(FitsTestCase):
         # section testing
         fs = fits.open(self.data('arange.fits'))
         with warnings.catch_warnings(record=True) as w:
-            assert fs[0].section[3, 2, 5] == np.array([357])
+            assert np.all(fs[0].section[3, 2, 5] == np.array([357]))
             assert len(w) == 0
