@@ -156,6 +156,9 @@ def parse_vowarning(line):
         result['is_exception'] = False
         result['is_other'] = False
         result['is_something'] = False
+        if not isinstance(line, unicode):
+            line = line.decode('utf-8')
+        result['message'] = line
 
     return result
 
@@ -855,11 +858,11 @@ class W39(VOTableSpecWarning):
 class W40(VOTableSpecWarning):
     """
     This is a terrible hack to support Simple Image Access Protocol
-    results from `<archive.noao.edu>`__.  It creates a field for the
-    coordinate projection type of type "double", which
-    actually contains character data.  We have to hack the field
-    to store character data, or we can't read it in.  A warning
-    will be raised when this happens.
+    results from `archive.noao.edu <http://archive.noao.edu>`__.  It
+    creates a field for the coordinate projection type of type "double",
+    which actually contains character data.  We have to hack the field
+    to store character data, or we can't read it in.  A warning will be
+    raised when this happens.
     """
 
     message = "'cprojection' datatype repaired"
@@ -1009,6 +1012,26 @@ class W51(VOTableSpecWarning):
     default_args = ('x', 'n-bit')
 
 
+class W52(VOTableSpecWarning):
+    """
+    The BINARY2 format was introduced in VOTable 1.3.  It should
+    not be present in files marked as an earlier version.
+    """
+
+    message = ("The BINARY2 format was introduced in VOTable 1.3, but "
+               "this file is declared as version '%s'")
+    default_args = ('1.2',)
+
+
+class W53(VOTableSpecWarning):
+    """
+    The VOTABLE element must contain at least one RESOURCE element.
+    """
+
+    message = ("VOTABLE element must contain at least one RESOURCE element.")
+    default_args = ()
+
+
 class E01(VOWarning, ValueError):
     """
     The size specifier for a ``char`` or ``unicode`` field must be
