@@ -227,7 +227,9 @@ def validate(source, output=sys.stdout, xmllint=False, filename=None):
             votable = parse(content_buffer, pedantic=False, filename=filename)
         except ValueError as e:
             lines.append(str(e))
-    lines = [str(x.message) for x in warning_lines] + lines
+
+    lines = [str(x.message) for x in warning_lines if
+             issubclass(x.category, exceptions.VOWarning)] + lines
 
     content_buffer.seek(0)
     output.write(u"Validation report for {0}\n\n".format(filename))
@@ -239,7 +241,7 @@ def validate(source, output=sys.stdout, xmllint=False, filename=None):
             w = exceptions.parse_vowarning(warning)
 
             if not w['is_something']:
-                output.write(warning)
+                output.write(w['message'])
                 output.write(u'\n\n')
             else:
                 line = xml_lines[w['nline'] - 1]
