@@ -19,6 +19,10 @@ from .utils.misc import find_current_module
 __all__ = ['log', 'AstropyLogger', 'LoggingError']
 
 
+# Initialize by calling _init_log()
+log = None
+
+
 class LoggingError(Exception):
     """
     This exception is for various errors that occur in the astropy logger,
@@ -36,7 +40,6 @@ class _AstLogIPYExc(Exception):
 
 
 # Read in configuration
-
 LOG_LEVEL = ConfigurationItem('log_level', 'INFO',
                               "Threshold for the logging messages. Logging "
                               "messages that are less severe than this level "
@@ -71,61 +74,55 @@ LOG_FILE_FORMAT = ConfigurationItem('log_file_format', "%(asctime)r, "
                                     "Format for log file entries")
 
 
-# The following function is copied from the source code of Python 2.7 and 3.2.
-# This function is not included in Python 2.6 and 3.1, so we have to include it
-# here to provide uniform behavior across versions.
+def _init_log():
+    """Initializes the Astropy log--in most circumstances this is called
+    automatically when importing astropy.
+    """
+
+    global log
+
+    orig_logger_cls = logging.getLoggerClass()
+    logging.setLoggerClass(AstropyLogger)
+    try:
+        log = logging.getLogger('astropy')
+        log._set_defaults()
+    finally:
+        logging.setLoggerClass(orig_logger_cls)
+
+    return log
 
 
-def _checkLevel(level):
-    '''
-    '''
-    if isinstance(level, int):
-        rv = level
-    elif str(level) == level:
-        if level not in logging._levelNames:
-            raise ValueError("Unknown level: %r" % level)
-        rv = logging._levelNames[level]
-    else:
-        raise TypeError("Level not an integer or a valid string: %r" % level)
-    return rv
+def _teardown_log():
+    """Shut down exception and warning logging (if enabled) and clear all
+    Astropy loggers from the logging module's cache.
 
+    This involves poking some logging module interals, so much if it is 'at
+    your own risk' and is allowed to pass silently if any exceptions occur.
+    """
 
-# We now have to be sure that we overload the setLevel in FileHandler, again
-# for compatibility with Python 2.6 and 3.1.
+    global log
 
+    if log.exception_logging_enabled():
+        log.disable_exception_logging()
 
-class FileHandler(logging.FileHandler):
-    def setLevel(self, level):
-        """
-        Set the logging level of this handler.
-        """
-        self.level = _checkLevel(level)
+    if log.warnings_logging_enabled():
+        log.disable_warnings_logging()
 
+    del log
 
-class FilterOrigin(object):
-    '''A filter for the record origin'''
-    def __init__(self, origin):
-        self.origin = origin
+    # Now for the fun stuff...
+    try:
+        logging._acquireLock()
+        try:
+            loggerDict = logging.Logger.manager.loggerDict
+            for key in loggerDict.keys():
+                if key == 'astropy' or key.startswith('astropy.'):
+                    del loggerDict[key]
+        finally:
+            logging._releaseLock()
+    except:
+        pass
 
-    def filter(self, record):
-        return record.origin.startswith(self.origin)
-
-
-class ListHandler(logging.Handler):
-    '''A handler that can be used to capture the records in a list'''
-
-    def __init__(self, filter_level=None, filter_origin=None):
-        logging.Handler.__init__(self)
-        self.log_list = []
-
-    def emit(self, record):
-        self.log_list.append(record)
-
-    def setLevel(self, level):
-        """
-        Set the logging level of this handler.
-        """
-        self.level = _checkLevel(level)
 
 Logger = logging.getLoggerClass()
 
@@ -239,10 +236,14 @@ class AstropyLogger(Logger):
     _excepthook_orig = None
 
     def _excepthook(self, etype, value, traceback):
-        tb = traceback
-        while tb.tb_next is not None:
-            tb = tb.tb_next
-        mod = inspect_getmodule(tb)
+
+        if traceback is None:
+            mod = None
+        else:
+            tb = traceback
+            while tb.tb_next is not None:
+                tb = tb.tb_next
+            mod = inspect_getmodule(tb)
 
         # include the the error type in the message.
         if len(value.args) > 0:
@@ -371,7 +372,7 @@ class AstropyLogger(Logger):
             color_print(record.levelname, 'brown', end='')
         else:
             color_print(record.levelname, 'red', end='')
-        print(": " + record.msg + " [{0:s}]".format(record.origin))
+        print(": {0} [{1:s}]".format(record.msg, record.origin))
 
     @contextmanager
     def log_to_file(self, filename, filter_level=None, filter_origin=None):
@@ -535,12 +536,56 @@ class AstropyLogger(Logger):
             self.enable_exception_logging()
 
 
-# Set up the class and initialize logger
-_orig_logger_cls = logging.getLoggerClass()
-logging.setLoggerClass(AstropyLogger)
-try:
-    log = logging.getLogger('astropy')
-    log._set_defaults()
-finally:
-    logging.setLoggerClass(_orig_logger_cls)
-    del _orig_logger_cls
+# The following function is copied from the source code of Python 2.7 and 3.2.
+# This function is not included in Python 2.6 and 3.1, so we have to include it
+# here to provide uniform behavior across versions.
+def _checkLevel(level):
+    '''
+    '''
+    if isinstance(level, int):
+        rv = level
+    elif str(level) == level:
+        if level not in logging._levelNames:
+            raise ValueError("Unknown level: %r" % level)
+        rv = logging._levelNames[level]
+    else:
+        raise TypeError("Level not an integer or a valid string: %r" % level)
+    return rv
+
+
+# We now have to be sure that we overload the setLevel in FileHandler, again
+# for compatibility with Python 2.6 and 3.1.
+
+
+class FileHandler(logging.FileHandler):
+    def setLevel(self, level):
+        """
+        Set the logging level of this handler.
+        """
+        self.level = _checkLevel(level)
+
+
+class FilterOrigin(object):
+    '''A filter for the record origin'''
+    def __init__(self, origin):
+        self.origin = origin
+
+    def filter(self, record):
+        return record.origin.startswith(self.origin)
+
+
+class ListHandler(logging.Handler):
+    '''A handler that can be used to capture the records in a list'''
+
+    def __init__(self, filter_level=None, filter_origin=None):
+        logging.Handler.__init__(self)
+        self.log_list = []
+
+    def emit(self, record):
+        self.log_list.append(record)
+
+    def setLevel(self, level):
+        """
+        Set the logging level of this handler.
+        """
+        self.level = _checkLevel(level)
