@@ -17,6 +17,8 @@ from ..utils.console import color_print
 from ..config import ConfigurationItem
 from  ..io import registry as io_registry
 
+PY3 = sys.version_info[0] >= 3
+
 # Python 2 and 3 source compatibility
 try:
     unicode
@@ -772,17 +774,36 @@ class Row(object):
     def __init__(self, table, index):
         self._table = table
         self._index = index
-        self._data = table._data[index]
+        try:
+            self._data = table._data[index]
 
-        # MaskedArray __getitem__ has a strange behavior where if a
-        # row mask is all False then it returns a np.void which
-        # has no mask attribute. This makes it impossible to then set
-        # the mask. Here we recast back to mvoid. This was fixed in
-        # Numpy following issue numpy/numpy#483, and the fix should be
-        # included in Numpy 1.8.0.
-        if self._table.masked and isinstance(self._data, np.void):
-            self._data = ma.core.mvoid(self._data,
-                                       mask=self._table._mask[index])
+            # MaskedArray __getitem__ has a strange behavior where if a
+            # row mask is all False then it returns a np.void which
+            # has no mask attribute. This makes it impossible to then set
+            # the mask. Here we recast back to mvoid. This was fixed in
+            # Numpy following issue numpy/numpy#483, and the fix should be
+            # included in Numpy 1.8.0.
+            if self._table.masked and isinstance(self._data, np.void):
+                self._data = ma.core.mvoid(self._data,
+                                           mask=self._table._mask[index])
+        except ValueError as err:
+            # Another bug (or maybe same?) that is fixed in 1.8 prevents accessing
+            # a row in masked array if it has object-type members.
+            # >>> x = np.ma.empty(1, dtype=[('a', 'O')])
+            # >>> x['a'] = 1
+            # >>> x['a'].mask = True
+            # >>> x[0]
+            # ValueError: Setting void-array with object members using buffer. [numpy.ma.core]
+            #
+            # All we do here is re-raise with a more informative message
+            from distutils import version
+            if (str(err).startswith('Setting void-array with object members')
+                    and version.LooseVersion(np.__version__) < version.LooseVersion('1.8')):
+                raise ValueError('Cannot access table row with Object type columns, due to '
+                                 'a bug in numpy {0}.  Please upgrade to numpy 1.8 or newer.'
+                                 .format(np.__version__))
+            else:
+                raise
 
     def __getitem__(self, item):
         return self.data[item]
@@ -1359,7 +1380,7 @@ class Table(object):
         else:
             raise StopIteration
 
-    if sys.version_info[0] < 3:  # pragma: py2
+    if not PY3:  # pragma: py2
         next = __next__
 
     def field(self, item):
@@ -1736,4 +1757,61 @@ class Table(object):
     @property
     def meta(self):
         return self._meta
-    
+
+    def __lt__(self, other):
+        if PY3:
+            return super(Table, self).__lt__(other)
+        else:
+            raise TypeError(
+                "unorderable types: Table() < {0}".format(str(type(other))))
+
+    def __gt__(self, other):
+        if PY3:
+            return super(Table, self).__gt__(other)
+        else:
+            raise TypeError(
+                "unorderable types: Table() > {0}".format(str(type(other))))
+
+    def __le__(self, other):
+        if PY3:
+            return super(Table, self).__le__(other)
+        else:
+            raise TypeError(
+                "unorderable types: Table() <= {0}".format(str(type(other))))
+
+    def __ge__(self, other):
+        if PY3:
+            return super(Table, self).__ge__(other)
+        else:
+            raise TypeError(
+                "unorderable types: Table() >= {0}".format(str(type(other))))
+
+    def __eq__(self, other):
+
+        if isinstance(other, Table):
+            other = other._data
+
+        if self.masked:
+            if isinstance(other, np.ma.MaskedArray):
+                result = self._data == other
+            else:
+                # If mask is True, then by definition the row doesn't match
+                # because the other array is not masked.
+                false_mask = np.zeros(1, dtype=[(n, bool)
+                                                for n in self.dtype.names])
+                result = (self._data.data == other) & (self.mask == false_mask)
+        else:
+            if isinstance(other, np.ma.MaskedArray):
+                # If mask is True, then by definition the row doesn't match
+                # because the other array is not masked.
+                false_mask = np.zeros(1, dtype=[(n, bool)
+                                                for n in other.dtype.names])
+                result = ((self._data == other.data) &
+                          (other.mask == false_mask))
+            else:
+                result = self._data == other
+
+        return result
+
+    def __ne__(self, other):
+        return ~self.__eq__(other)
