@@ -57,6 +57,12 @@ class SetupData(object):
                 self._d = Column(name='d', data=[7, 8, 7])
             return self._d
 
+    @property
+    def obj(self):
+        if Column is not None and not hasattr(self, '_obj'):
+            self._obj = Column(name='obj', data=[1, 'string', 3], dtype='O')
+            return self._obj
+
     @property
     def t(self):
         if Table is not None:
@@ -389,24 +395,28 @@ class TestAddRow(SetupData):
             return self._t
 
     def test_add_none_to_empty_table(self):
-        t = Table(names=('a', 'b'), dtypes=('i', 'S4'))
+        t = Table(names=('a', 'b', 'c'), dtypes=('i', 'S4', 'O'))
         t.add_row()
         assert t['a'][0] == 0
         assert t['b'][0] == b''
+        assert t['c'][0] == 0
         t.add_row()
         assert t['a'][1] == 0
         assert t['b'][1] == b''
+        assert t['c'][1] == 0
 
     def test_add_stuff_to_empty_table(self):
-        t = Table(names=('a', 'b'), dtypes=('i', 'S8'))
-        t.add_row([1, 'hello'])
+        t = Table(names=('a', 'b', 'obj'), dtypes=('i', 'S8', 'O'))
+        t.add_row([1, 'hello', 'world'])
         assert t['a'][0] == 1
         assert t['b'][0] == b'hello'
+        assert t['obj'][0] == 'world'
         # Make sure it is not repeating last row but instead
         # adding zeros (as documented)
         t.add_row()
         assert t['a'][1] == 0
         assert t['b'][1] == b''
+        assert t['obj'][1] == 0
 
     def test_add_table_row(self):
         t = self.t
@@ -417,6 +427,14 @@ class TestAddRow(SetupData):
         assert np.allclose(t['b'], np.array([4.0, 5.1, 6.2, 4.0]))
         assert np.all(t['c'] == np.array(['7', '8', '9', '7']))
 
+    def test_add_table_row_obj(self):
+        t = Table([self.a, self.b, self.obj])
+        t.add_row([1, 4.0, [10]])
+        assert len(t) == 4
+        assert np.all(t['a'] == np.array([1, 2, 3, 1]))
+        assert np.allclose(t['b'], np.array([4.0, 5.1, 6.2, 4.0]))
+        assert np.all(t['obj'] == np.array([1, 'string', 3, [10]], dtype='O'))
+
     def test_add_with_tuple(self):
         t = self.t
         t.add_row((4, 7.2, '1'))
@@ -696,3 +714,159 @@ class TestConvertNumpyArray():
 
         with pytest.raises(ValueError):
             np_data = np.array(d, dtype=[('c', 'i8'), ('d', 'i8')])
+
+
+def test_disallow_inequality_comparisons():
+    """
+    Regression test for #828 - disallow comparison operators on whole Table
+    """
+
+    t = table.Table()
+
+    with pytest.raises(TypeError):
+        t > 2
+
+    with pytest.raises(TypeError):
+        t < 1.1
+
+    with pytest.raises(TypeError):
+        t >= 5.5
+
+    with pytest.raises(TypeError):
+        t <= -1.1
+
+
+def test_equality():
+
+    t = table.Table.read([' a b  c  d',
+                          ' 2 c 7.0 0',
+                          ' 2 b 5.0 1',
+                          ' 2 b 6.0 2',
+                          ' 2 a 4.0 3',
+                          ' 0 a 0.0 4',
+                          ' 1 b 3.0 5',
+                          ' 1 a 2.0 6',
+                          ' 1 a 1.0 7',
+                         ], format='ascii')
+
+    # All rows are equal
+    assert np.all(t==t)
+
+    # Assert no rows are different
+    assert not np.any(t!=t)
+
+    # Check equality result for a given row
+    assert np.all((t == t[3]) == np.array([0,0,0,1,0,0,0,0], dtype=bool))
+
+    # Check inequality result for a given row
+    assert np.all((t != t[3]) == np.array([1,1,1,0,1,1,1,1], dtype=bool))
+
+    t2 = table.Table.read([' a b  c  d',
+                           ' 2 c 7.0 0',
+                           ' 2 b 5.0 1',
+                           ' 3 b 6.0 2',
+                           ' 2 a 4.0 3',
+                           ' 0 a 1.0 4',
+                           ' 1 b 3.0 5',
+                           ' 1 c 2.0 6',
+                           ' 1 a 1.0 7',
+                          ], format='ascii')
+
+    # In the above cases, Row.__eq__ gets called, but now need to make sure
+    # Table.__eq__ also gets called.
+    assert np.all((t == t2) == np.array([1,1,0,1,0,1,0,1], dtype=bool))
+    assert np.all((t != t2) == np.array([0,0,1,0,1,0,1,0], dtype=bool))
+
+    # Check that comparing to a structured array works
+    assert np.all((t == t2._data) == np.array([1,1,0,1,0,1,0,1], dtype=bool))
+    assert np.all((t._data == t2) == np.array([1,1,0,1,0,1,0,1], dtype=bool))
+
+
+@pytest.mark.xfail('numpy_lt_1p5')
+def test_equality_masked():
+
+    t = table.Table.read([' a b  c  d',
+                          ' 2 c 7.0 0',
+                          ' 2 b 5.0 1',
+                          ' 2 b 6.0 2',
+                          ' 2 a 4.0 3',
+                          ' 0 a 0.0 4',
+                          ' 1 b 3.0 5',
+                          ' 1 a 2.0 6',
+                          ' 1 a 1.0 7',
+                         ], format='ascii')
+
+    # Make into masked table
+    t = table.Table(t, masked=True)
+
+    # All rows are equal
+    assert np.all(t==t)
+
+    # Assert no rows are different
+    assert not np.any(t!=t)
+
+    # Check equality result for a given row
+    assert np.all((t == t[3]) == np.array([0,0,0,1,0,0,0,0], dtype=bool))
+
+    # Check inequality result for a given row
+    assert np.all((t != t[3]) == np.array([1,1,1,0,1,1,1,1], dtype=bool))
+
+    t2 = table.Table.read([' a b  c  d',
+                           ' 2 c 7.0 0',
+                           ' 2 b 5.0 1',
+                           ' 3 b 6.0 2',
+                           ' 2 a 4.0 3',
+                           ' 0 a 1.0 4',
+                           ' 1 b 3.0 5',
+                           ' 1 c 2.0 6',
+                           ' 1 a 1.0 7',
+                          ], format='ascii')
+
+    # In the above cases, Row.__eq__ gets called, but now need to make sure
+    # Table.__eq__ also gets called.
+    assert np.all((t == t2) == np.array([1,1,0,1,0,1,0,1], dtype=bool))
+    assert np.all((t != t2) == np.array([0,0,1,0,1,0,1,0], dtype=bool))
+
+    # Check that masking a value causes the row to differ
+    t.mask['a'][0] = True
+    assert np.all((t == t2) == np.array([0,1,0,1,0,1,0,1], dtype=bool))
+    assert np.all((t != t2) == np.array([1,0,1,0,1,0,1,0], dtype=bool))
+
+    # Check that comparing to a structured array works
+    assert np.all((t == t2._data) == np.array([0,1,0,1,0,1,0,1], dtype=bool))
+
+
+@pytest.mark.xfail
+def test_equality_masked_bug():
+    """
+    This highlights a Numpy bug. Once it works, it can be moved into the
+    test_equality_masked test. Related Numpy bug report:
+
+      https://github.com/numpy/numpy/issues/3840
+    """
+
+    t = table.Table.read([' a b  c  d',
+                          ' 2 c 7.0 0',
+                          ' 2 b 5.0 1',
+                          ' 2 b 6.0 2',
+                          ' 2 a 4.0 3',
+                          ' 0 a 0.0 4',
+                          ' 1 b 3.0 5',
+                          ' 1 a 2.0 6',
+                          ' 1 a 1.0 7',
+                         ], format='ascii')
+
+    t = table.Table(t, masked=True)
+
+    t2 = table.Table.read([' a b  c  d',
+                           ' 2 c 7.0 0',
+                           ' 2 b 5.0 1',
+                           ' 3 b 6.0 2',
+                           ' 2 a 4.0 3',
+                           ' 0 a 1.0 4',
+                           ' 1 b 3.0 5',
+                           ' 1 c 2.0 6',
+                           ' 1 a 1.0 7',
+                          ], format='ascii')
+
+    assert np.all((t._data == t2) == np.array([0,1,0,1,0,1,0,1], dtype=bool))
