@@ -1,9 +1,10 @@
+# Licensed under a 3-clause BSD style license - see LICENSE.rst
 import sys
 import warnings
 
-import pytest
+from .helper import pytest
 from .. import log
-from ..logger import LoggingError
+from ..logger import LoggingError, LOG_LEVEL
 
 
 # Save original values of hooks. These are not the system values, but the
@@ -13,6 +14,12 @@ _excepthook = sys.__excepthook__
 _showwarning = warnings.showwarning
 
 
+try:
+    ip = get_ipython()
+except NameError:
+    ip = None
+
+
 def setup_function(function):
 
     # Reset hooks to original values
@@ -109,8 +116,8 @@ def test_warnings_logging_with_custom_class():
     assert log_list[0].origin == 'astropy.tests.test_logger'
 
 
-def test_warning_logging_with_io_vo_warning():
-    from ..io.vo.exceptions import W02, vo_warn
+def test_warning_logging_with_io_votable_warning():
+    from ..io.votable.exceptions import W02, vo_warn
 
     with warnings.catch_warnings(record=True) as warn_list:
         log.enable_warnings_logging()
@@ -138,6 +145,9 @@ def test_exception_logging_enable_twice():
     assert e.value.args[0] == 'Exception logging has already been enabled'
 
 
+# You can't really override the exception handler in IPython this way, so
+# this test doesn't really make sense in the IPython context.
+@pytest.mark.skipif("ip is not None")
 def test_exception_logging_overridden():
     log.enable_exception_logging()
     sys.excepthook = lambda: None
@@ -146,6 +156,7 @@ def test_exception_logging_overridden():
     assert e.value.args[0] == 'Cannot disable exception logging: sys.excepthook was not set by this logger, or has been overridden'
 
 
+@pytest.mark.xfail("ip is not None")
 def test_exception_logging():
 
     # Without exception logging
@@ -187,6 +198,7 @@ def test_exception_logging():
     assert len(log_list) == 0
 
 
+@pytest.mark.xfail("ip is not None")
 def test_exception_logging_origin():
     # The point here is to get an exception raised from another location
     # and make sure the error's origin is reported correctly
@@ -214,19 +226,28 @@ def test_exception_logging_origin():
 @pytest.mark.parametrize(('level'), [None, 'DEBUG', 'INFO', 'WARN', 'ERROR'])
 def test_log_to_list(level):
 
-    if level is not None:
-        log.setLevel(level)
+    orig_level = log.level
 
-    with log.log_to_list() as log_list:
-        log.error("Error message")
-        log.warn("Warning message")
-        log.info("Information message")
-        log.debug("Debug message")
+    try:
+        if level is not None:
+            log.setLevel(level)
+
+        with log.log_to_list() as log_list:
+            log.error("Error message")
+            log.warning("Warning message")
+            log.info("Information message")
+            log.debug("Debug message")
+    finally:
+        log.setLevel(orig_level)
+
+    if level is None:
+        # The log level *should* be set to whatever it was in the config
+        level = LOG_LEVEL()
 
     # Check list length
     if level == 'DEBUG':
         assert len(log_list) == 4
-    elif level is None or level == 'INFO':
+    elif level == 'INFO':
         assert len(log_list) == 3
     elif level == 'WARN':
         assert len(log_list) == 2
@@ -259,7 +280,7 @@ def test_log_to_list_level():
 
     with log.log_to_list(filter_level='ERROR') as log_list:
         log.error("Error message")
-        log.warn("Warning message")
+        log.warning("Warning message")
 
     assert len(log_list) == 1 and log_list[0].levelname == 'ERROR'
 
@@ -268,7 +289,7 @@ def test_log_to_list_origin1():
 
     with log.log_to_list(filter_origin='astropy.tests') as log_list:
         log.error("Error message")
-        log.warn("Warning message")
+        log.warning("Warning message")
 
     assert len(log_list) == 2
 
@@ -277,7 +298,7 @@ def test_log_to_list_origin2():
 
     with log.log_to_list(filter_origin='astropy.wcs') as log_list:
         log.error("Error message")
-        log.warn("Warning message")
+        log.warning("Warning message")
 
     assert len(log_list) == 0
 
@@ -288,26 +309,34 @@ def test_log_to_file(tmpdir, level):
     local_path = tmpdir.join('test.log')
     log_file = local_path.open('wb')
     log_path = str(local_path.realpath())
+    orig_level = log.level
 
-    if level is not None:
-        log.setLevel(level)
+    try:
+        if level is not None:
+            log.setLevel(level)
 
-    with log.log_to_file(log_path):
-        log.error("Error message")
-        log.warn("Warning message")
-        log.info("Information message")
-        log.debug("Debug message")
+        with log.log_to_file(log_path):
+            log.error("Error message")
+            log.warning("Warning message")
+            log.info("Information message")
+            log.debug("Debug message")
 
-    log_file.close()
+        log_file.close()
+    finally:
+        log.setLevel(orig_level)
 
     log_file = local_path.open('rb')
     log_entries = log_file.readlines()
     log_file.close()
 
+    if level is None:
+        # The log level *should* be set to whatever it was in the config
+        level = LOG_LEVEL()
+
     # Check list length
     if level == 'DEBUG':
         assert len(log_entries) == 4
-    elif level is None or level == 'INFO':
+    elif level == 'INFO':
         assert len(log_entries) == 3
     elif level == 'WARN':
         assert len(log_entries) == 2
@@ -336,7 +365,7 @@ def test_log_to_file_level(tmpdir):
 
     with log.log_to_file(log_path, filter_level='ERROR'):
         log.error("Error message")
-        log.warn("Warning message")
+        log.warning("Warning message")
 
     log_file.close()
 
@@ -355,7 +384,7 @@ def test_log_to_file_origin1(tmpdir):
 
     with log.log_to_file(log_path, filter_origin='astropy.tests'):
         log.error("Error message")
-        log.warn("Warning message")
+        log.warning("Warning message")
 
     log_file.close()
 
@@ -374,7 +403,7 @@ def test_log_to_file_origin2(tmpdir):
 
     with log.log_to_file(log_path, filter_origin='astropy.wcs'):
         log.error("Error message")
-        log.warn("Warning message")
+        log.warning("Warning message")
 
     log_file.close()
 
