@@ -1,7 +1,6 @@
 import sys
 
-import pytest
-
+from ...tests.helper import pytest
 from ...utils.data import get_pkg_data_filename
 
 fd = None
