@@ -18,6 +18,7 @@ import numpy as np
 from numpy import ma
 
 from ..utils.compat.fractions import Fraction
+from .utils import is_effectively_unity
 from . import format as unit_format
 
 # TODO: Support functional units, e.g. log(x), ln(x)
@@ -129,12 +130,17 @@ class UnitBase(object):
             return CompositeUnit(1, [self, m], [1, -1])._simplify()
         elif isinstance(m, Quantity):
             return Quantity(1, self) / m
+        elif isinstance(m, basestring):
+            return self / Unit(m)
         else:
             return Quantity(1. / m, self)
 
     def __rdiv__(self, m):
         from .quantity import Quantity
-        return Quantity(m, CompositeUnit(1.0, [self], [-1])._simplify())
+        if isinstance(m, basestring):
+            return Unit(m) / Quantity(1, self)
+        else:
+            return Quantity(m, CompositeUnit(1.0, [self], [-1])._simplify())
 
     def __truediv__(self, m):
         return self.__div__(m)
@@ -148,14 +154,19 @@ class UnitBase(object):
             return CompositeUnit(1, [self, m], [1, 1])._simplify()
         elif isinstance(m, Quantity):
             return Quantity(1, self) * m
+        elif isinstance(m, basestring):
+            return self * Unit(m)
         else:
             return Quantity(m, self)
 
     def __rmul__(self, m):
         from .quantity import Quantity
-        return Quantity(m, self)
+        if isinstance(m, basestring):
+            return Unit(m) * Quantity(1, self)
+        else:
+            return Quantity(m, self)
 
-    if sys.version_info[0] >= 3:
+    if sys.version_info[0] >= 3:  # pragma: no cover
         def __hash__(self):
             # Since this class defines __eq__, it will become unhashable
             # on Python 3.x, so we need to define our own hash.
@@ -365,7 +376,7 @@ class UnitBase(object):
         return self.to(
             other, value=value, equivalencies=equivalencies)
 
-    def decompose(self, bases=[]):
+    def decompose(self, bases=set()):
         """
         Return a unit object composed of only irreducible units.
 
@@ -511,7 +522,8 @@ class UnitBase(object):
 
         return [self]
 
-    def compose(self, equivalencies=[], units=None, max_depth=2):
+    def compose(self, equivalencies=[], units=None, max_depth=2,
+                include_prefix_units=False):
         """
         Return the simplest possible composite unit(s) that represent
         the given unit.  Since there may be multiple equally simple
@@ -532,6 +544,10 @@ class UnitBase(object):
             The maximum recursion depth to use when composing into
             composite units.
 
+        include_prefix_units : bool, optional
+            When `True`, include prefixed units in the result.
+            Default is `False`.
+
         Returns
         -------
         units : list of `CompositeUnit`
@@ -544,16 +560,17 @@ class UnitBase(object):
             filtered_namespace = set()
             for tunit in units:
                 if (isinstance(tunit, UnitBase) and
-                        not isinstance(tunit, PrefixUnit)):
+                    (include_prefix_units or
+                     not isinstance(tunit, PrefixUnit))):
                     filtered_namespace.add(tunit)
             return filtered_namespace
 
         if units is None:
             units = filter_units(UnitBase._registry.values())
         elif isinstance(units, dict):
-            units = set(units.values())
+            units = set(filter_units(units.itervalues()))
         elif inspect.ismodule(units):
-            units = filter_units(vars(units).values())
+            units = filter_units(vars(units).itervalues())
         else:
             units = set(units)
 
@@ -586,11 +603,19 @@ class UnitBase(object):
         bases = set(system.bases)
 
         def score(compose):
-            sum = 0
-            for base in compose._bases:
-                if base in bases:
-                    sum += 1
-            return sum / float(len(compose._bases))
+            # In case that compose._bases has no elements we return
+            # 'np.inf' as 'score value'.  It does not really matter which
+            # number we would return. This case occurs for instance for
+            # dimensionless quantities:
+            if len(compose._bases) == 0:
+                return np.inf
+            else:
+                sum = 0
+                for base in compose._bases:
+                    if base in bases:
+                        sum += 1
+
+                return sum / float(len(compose._bases))
 
         x = self.decompose(bases=bases)
         composed = x.compose(units=system)
@@ -641,7 +666,8 @@ class UnitBase(object):
                          [']'])
                 return '\n'.join(lines)
 
-    def find_equivalent_units(self, equivalencies=[], units=None):
+    def find_equivalent_units(self, equivalencies=[], units=None,
+                              include_prefix_units=False):
         """
         Return a list of all the units that are the same type as the
         specified unit.
@@ -660,6 +686,10 @@ class UnitBase(object):
             equivalencies.  Otherwise, may be a dict, module or
             sequence containing the units to search for equivalencies.
 
+        include_prefix_units : bool, optional
+            When `True`, include prefixed units in the result.
+            Default is `False`.
+
         Returns
         -------
         units : list of `UnitBase`
@@ -668,7 +698,8 @@ class UnitBase(object):
             pretty-prints the list of units when output.
         """
         results = self.compose(
-            equivalencies=equivalencies, units=units, max_depth=1)
+            equivalencies=equivalencies, units=units, max_depth=1,
+            include_prefix_units=include_prefix_units)
         results = [
             x._bases[0] for x in results if len(x._bases) == 1]
         return self.EquivalentUnitsList(results)
@@ -803,6 +834,9 @@ class NamedUnit(UnitBase):
         if not self._names:
             raise UnitsException("unit has no string representation")
 
+        # Loop through all of the names first, to ensure of all them
+        # are new, then add them all as a single "transaction" below.
+        cls = self.__class__
         for st in self._names:
             if not re.match("^[A-Za-z_]+$", st):
                 # will cause problems for simple string parser in
@@ -810,17 +844,31 @@ class NamedUnit(UnitBase):
                 raise ValueError(
                     "Invalid unit name {0!r}".format(st))
 
+            if (register and st in self._namespace or (st in cls._registry and
+                    cls._registry[st] != self)):
+                raise ValueError(
+                    "Object with name {0!r} already exists "
+                    "in namespace.".format(st))
+
+        for st in self._names:
             if register:
-                if st in self._namespace:
-                    raise ValueError(
-                        "Object with name {0!r} already exists "
-                        "in namespace ({1})".format(
-                            st, self._namespace[st].__doc__))
                 self._namespace[st] = self
 
             self._registry[st] = self
 
 
+def _recreate_irreducible_unit(names, registered):
+    """
+    This is used to reconstruct units when passed around by
+    multiprocessing.
+    """
+    namespace = UnitBase._get_namespace()
+    if names[0] in namespace:
+        return namespace[names[0]]
+    else:
+        return IrreducibleUnit(names, register=registered)
+
+
 class IrreducibleUnit(NamedUnit):
     """
     Irreducible units are the units that all other units are defined
@@ -829,6 +877,18 @@ class IrreducibleUnit(NamedUnit):
     Examples are meters, seconds, kilograms, amperes, etc.  There is
     only once instance of such a unit per type.
     """
+    def __reduce__(self):
+        # When IrreducibleUnit objects are passed to other processes
+        # over multiprocessing, they need to be recreated to be the
+        # ones already in the subprocesses' namespace, not new
+        # objects, or they will be considered "unconvertible".
+        # Therefore, we have a custom pickler/unpickler that
+        # understands how to recreate the Unit on the other side.
+        namespace = UnitBase._get_namespace()
+        return (_recreate_irreducible_unit,
+                (self.names, self.names[0] in namespace),
+                self.__dict__)
+
     @property
     def scale(self):
         """
@@ -850,9 +910,7 @@ class IrreducibleUnit(NamedUnit):
         """
         return [1.0]
 
-    def decompose(self, bases=[]):
-        bases = set(bases)
-
+    def decompose(self, bases=set()):
         if len(bases) and not self in bases:
             for base in bases:
                 if self.is_equivalent(base):
@@ -958,7 +1016,6 @@ class _UnitMetaClass(type):
             # __init__ on the Unit class.
             return super(_UnitMetaClass, self).__call__(
                 s, represents, format=format, register=register, doc=doc)
-            raise TypeError("Can not convert {0!r} to a unit".format(s))
 
         elif isinstance(s, UnitBase):
             return s
@@ -974,23 +1031,26 @@ class _UnitMetaClass(type):
             f = unit_format.get_format(format)
             try:
                 return f.parse(s)
-            except ValueError as e:
-                if parse_strict == 'raise':
-                    raise
-                elif parse_strict == 'warn':
-                    warnings.warn(
-                        "'{0}' did not parse using format '{1}'. {2}".format(
-                            s, format, str(e)), UnitsWarning)
-                elif parse_strict != 'silent':
-                    raise ValueError(
-                        "'parse_strict' must be 'warn', 'raise' or 'silent'")
+            except Exception as e:
+                if parse_strict == 'silent':
+                    pass
+                else:
+                    msg = "'{0}' did not parse as unit format '{1}': {2}".format(
+                        s, format, str(e))
+                    if parse_strict == 'raise':
+                        raise ValueError(msg)
+                    elif parse_strict == 'warn':
+                        warnings.warn(msg, UnitsWarning)
+                    else:
+                        raise ValueError(
+                            "'parse_strict' must be 'warn', 'raise' or 'silent'")
                 return UnrecognizedUnit(s)
 
         elif isinstance(s, (int, float, np.floating, np.integer)):
             return CompositeUnit(s, [], [])
 
         elif s is None:
-            raise ValueError("None is not a valid Unit")
+            raise TypeError("None is not a valid Unit")
 
         else:
             raise TypeError("{0} can not be converted to a Unit".format(s))
@@ -1090,7 +1150,7 @@ class Unit(NamedUnit):
         NamedUnit.__init__(self, st, register=register, doc=doc,
                            format=format)
 
-    def decompose(self, bases=[]):
+    def decompose(self, bases=set()):
         return self._represents.decompose(bases=bases)
     decompose.__doc__ = UnitBase.decompose.__doc__
 
@@ -1128,7 +1188,7 @@ class CompositeUnit(UnitBase):
         of the base units.
     """
     def __init__(self, scale, bases, powers):
-        if scale == 1.:
+        if scale == 1. or is_effectively_unity(scale):
             scale = 1
         self._scale = scale
         for base in bases:
@@ -1136,6 +1196,7 @@ class CompositeUnit(UnitBase):
                 raise TypeError("bases must be sequence of UnitBase instances")
         self._bases = bases
         self._powers = powers
+        self._decomposed_cache = None
 
     def __repr__(self):
         if len(self._bases):
@@ -1173,7 +1234,7 @@ class CompositeUnit(UnitBase):
         """
         return self._powers
 
-    def _expand_and_gather(self, decompose=False, bases=[]):
+    def _expand_and_gather(self, decompose=False, bases=set()):
         def add_unit(unit, power, scale):
             if len(bases) and unit not in bases:
                 for base in bases:
@@ -1182,7 +1243,10 @@ class CompositeUnit(UnitBase):
                         unit = base
                         break
 
-            new_parts[unit] = power + new_parts.get(unit, 0)
+            if unit in new_parts:
+                new_parts[unit] += power
+            else:
+                new_parts[unit] = power
             return scale
 
         new_parts = {}
@@ -1193,17 +1257,21 @@ class CompositeUnit(UnitBase):
                 b = b.decompose(bases=bases)
 
             if isinstance(b, CompositeUnit):
-                scale *= b.scale ** p
-                for b_sub, p_sub in zip(b.bases, b.powers):
+                scale *= b._scale ** p
+                for b_sub, p_sub in zip(b._bases, b._powers):
                     scale = add_unit(b_sub, p_sub * p, scale)
             else:
                 scale = add_unit(b, p, scale)
 
-        new_parts = [(b, p) for (b, p) in new_parts.items() if p != 0]
+        new_parts = [x for x in new_parts.items() if x[1] != 0]
         new_parts.sort(key=lambda x: x[1], reverse=True)
 
         self._bases = [x[0] for x in new_parts]
         self._powers = [x[1] for x in new_parts]
+
+        if is_effectively_unity(scale):
+            scale = 1
+
         self._scale = scale
 
     def __copy__(self):
@@ -1217,16 +1285,23 @@ class CompositeUnit(UnitBase):
         return self
     _simplify.__doc__ = UnitBase._simplify.__doc__
 
-    def decompose(self, bases=[]):
+    def decompose(self, bases=set()):
+        if len(bases) == 0 and self._decomposed_cache is not None:
+            return self._decomposed_cache
+
         for base in self.bases:
             if (not isinstance(base, IrreducibleUnit) or
                     (len(bases) and base not in bases)):
                 break
         else:
+            if len(bases) == 0:
+                self._decomposed_cache = self
             return self
 
         x = CompositeUnit(self.scale, self.bases, self.powers)
         x._expand_and_gather(True, bases=bases)
+        if len(bases) == 0:
+            self._decomposed_cache = x
         return x
     decompose.__doc__ = UnitBase.decompose.__doc__
 
