@@ -23,12 +23,11 @@ class Fits(generic.Generic):
     name = 'fits'
 
     def __init__(self):
+        super(Fits, self).__init__()
+
         if not '_units' in Fits.__dict__:
             Fits._units, Fits._deprecated_units = self._generate_unit_names()
 
-        if not '_parser' in Fits.__dict__:
-            Fits._parser = self._make_parser()
-
     @staticmethod
     def _generate_unit_names():
         import keyword
@@ -73,12 +72,9 @@ class Fits(generic.Generic):
         return names, deprecated_names
 
     @classmethod
-    def _parse_unit(cls, s, loc, toks):
-        from astropy.extern import pyparsing as p
-
-        unit = toks[0]
+    def _parse_unit(cls, unit):
         if unit not in cls._units:
-            raise p.ParseException(
+            raise ValueError(
                 "Unit {0!r} not supported by the FITS standard.".format(unit))
 
         if unit in cls._deprecated_units:
@@ -111,7 +107,7 @@ class Fits(generic.Generic):
         unit = utils.decompose_to_known_units(unit, self._get_unit_name)
 
         if isinstance(unit, core.CompositeUnit):
-            if unit.scale != 1.0:
+            if unit.scale != 1:
                 raise ValueError(
                     "The FITS unit format is not able to represent scale. "
                     "Multiply your data by {0:e}.".format(unit.scale))
