@@ -19,13 +19,12 @@ class VOUnit(generic.Generic):
     <http://www.ivoa.net/Documents/VOUnits/>`_.
     """
     def __init__(self):
+        super(VOUnit, self).__init__()
+
         if not '_units' in VOUnit.__dict__:
             unit_names = VOUnit._generate_unit_names()
             VOUnit._units, VOUnit._deprecated_units = unit_names
 
-        if not '_parser' in VOUnit.__dict__:
-            VOUnit._parser = self._make_parser()
-
     @staticmethod
     def _generate_unit_names():
         import keyword
@@ -65,13 +64,9 @@ class VOUnit(generic.Generic):
         return names, deprecated_names
 
     @classmethod
-    def _parse_unit(cls, s, loc, toks):
-        from astropy.extern import pyparsing as p
-
-        unit = toks[0]
-
+    def _parse_unit(cls, unit):
         if unit not in cls._units:
-            raise p.ParseException(
+            raise ValueError(
                 "Unit {0!r} not supported by the VOUnit "
                 "standard.".format(unit))
 
@@ -115,8 +110,6 @@ class VOUnit(generic.Generic):
                     if not ex.startswith('-'):
                         s += '+'
                     s += ex
-            else:
-                s = ''
 
             pairs = zip(unit.bases, unit.powers)
             pairs.sort(key=lambda x: x[1], reverse=True)
