@@ -15,6 +15,7 @@ from __future__ import (absolute_import, division, print_function,
 from . import core
 from . import si
 from . import astrophys
+from . import cgs
 
 
 __all__ = ['def_physical_type', 'get_physical_type']
@@ -61,8 +62,8 @@ def def_physical_type(unit, name):
     r = _tuple_repr(unit)
     if r in _physical_unit_mapping:
         raise ValueError(
-            "{0!r} already defined as {1!r}".format(
-                r, _physical_unit_mapping[r]))
+            "{0!r} ({1!r}) already defined as {2!r}".format(
+                r, name, _physical_unit_mapping[r]))
     _physical_unit_mapping[r] = name
 
 
@@ -92,6 +93,7 @@ for unit, name in [
     (si.m, 'length'),
     (si.m ** 2, 'area'),
     (si.m ** 3, 'volume'),
+    (si.s, 'time'),
     (si.rad, 'angle'),
     (si.sr, 'solid angle'),
     (si.m / si.s, 'speed'),
@@ -104,22 +106,42 @@ for unit, name in [
     (si.J, 'energy'),
     (si.Pa, 'pressure'),
     (si.W, 'power'),
+    (si.kg / si.m ** 3, 'mass density'),
+    (si.m ** 3 / si.kg, 'specific volume'),
+    (si.mol / si.m ** 3, 'molar volume'),
+    (si.kg * si.m / si.s, 'momentum/impulse'),
+    (si.kg * si.m ** 2 / si.s, 'angular momentum'),
+    (si.rad / si.s, 'angular speed'),
+    (si.rad / si.s ** 2, 'angular acceleration'),
     (si.g / (si.m * si.s), 'dynamic viscosity'),
     (si.m ** 2 / si.s, 'kinematic viscosity'),
     (si.m ** -1, 'wavenumber'),
+    (si.A, 'electrical current'),
     (si.C, 'electrical charge'),
     (si.V, 'electrical potential'),
     (si.Ohm, 'electrical resistance'),
     (si.S, 'electrical conductance'),
     (si.F, 'electrical capacitance'),
     (si.C * si.m, 'electrical dipole moment'),
+    (si.A / si.m ** 2, 'electrical current density'),
+    (si.V / si.m, 'electrical field strength'),
+    (si.C / si.m ** 2, 'electrical flux density'),
+    (si.C / si.m ** 3, 'electrical charge density'),
+    (si.F / si.m, 'permittivity'),
     (si.Wb, 'magnetic flux'),
     (si.T, 'magnetic flux density'),
+    (si.A / si.m, 'magnetic field strength'),
+    (si.H / si.m, 'electromagnetic field strength'),
     (si.H, 'inductance'),
     (si.cd, 'luminous intensity'),
     (si.lm, 'luminous flux'),
-    (si.lx, 'luminous emittence'),
+    (si.lx, 'luminous emittence/illuminance'),
+    (si.W / si.sr, 'radiant intensity'),
+    (si.cd / si.m ** 2, 'luminance'),
     (astrophys.Jy, 'spectral flux density'),
+    (cgs.erg / si.angstrom / si.cm ** 2 / si.s, 'spectral flux density wav'),
     (astrophys.R, 'photon flux'),
+    (astrophys.bit, 'data quantity'),
+    (astrophys.bit / si.s, 'bandwidth')
 ]:
     def_physical_type(unit, name)
