@@ -12,6 +12,7 @@ from ...tests.compat import assert_allclose
 from ... import units as u
 from .. import core
 from .. import format
+from ..utils import is_effectively_unity
 from ... import wcs
 
 
@@ -27,11 +28,11 @@ def test_unit_grammar():
         (["m**2", "m2", "m**(2)", "m**+2", "m+2", "m^(+2)"], u.m ** 2),
         (["m**-3", "m-3", "m^(-3)", "/m3"], u.m ** -3),
         (["m**(1.5)", "m(3/2)", "m**(3/2)", "m^(3/2)"], u.m ** 1.5),
-        (["2.54cm"], u.Unit(u.cm * 2.54)),
+        (["2.54 cm"], u.Unit(u.cm * 2.54)),
         (["10+8m"], u.Unit(u.m * 1e8)),
         # This is the VOUnits documentation, but doesn't seem to follow the
         # unity grammar (["3.45 10**(-4)Jy"], 3.45 * 1e-4 * u.Jy)
-        (["sqrt(m)"], u.m ** -2)
+        (["sqrt(m)"], u.m ** 0.5)
     ]
 
     for strings, unit in data:
@@ -48,13 +49,25 @@ def test_cds_grammar():
     data = [
         (["0.1nm"], u.AA),
         (["mW/m2"], u.Unit(u.erg / u.cm ** 2 / u.s)),
+        (["mW/(m2)"], u.Unit(u.erg / u.cm ** 2 / u.s)),
         (["km/s", "km.s-1"], u.km / u.s),
         (["10pix/nm"], u.Unit(10 * u.pix / u.nm)),
         (["1.5x10+11m"], u.Unit(1.5e11 * u.m)),
         (["m2"], u.m ** 2),
         (["10+21m"], u.Unit(u.m * 1e21)),
         (["2.54cm"], u.Unit(u.cm * 2.54)),
-        (["20%"], 0.20)]
+        (["20%"], 0.20),
+        (["ma"], u.ma),
+        (["mAU"], u.mAU),
+        (["uarcmin"], u.uarcmin),
+        (["uarcsec"], u.uarcsec),
+        (["kbarn"], u.kbarn),
+        (["Gbit"], u.Gbit),
+        (["kbyte"], u.kbyte),
+        (["mRy"], 0.001 * u.Ry),
+        (["mmag"], u.mmag),
+        (["Mpc"], u.Mpc),
+        (["Gyr"], u.Gyr)]
 
     for strings, unit in data:
         for s in strings:
@@ -68,7 +81,8 @@ def test_cds_grammar_fail():
         format.CDS().parse(s)
 
     data = ['0.1 nm', 'solMass(3/2)', 'km / s', 'km s-1',
-            'pix0.1nm', 'pix/(0.1nm)', 'km*s', 'km**2']
+            'pix0.1nm', 'pix/(0.1nm)', 'km*s', 'km**2',
+            '5x8+3m']
 
     for s in data:
         yield _test_cds_grammar_fail, s
@@ -159,7 +173,7 @@ def test_wcs_parse():
 
 
 def test_flatten_to_known():
-    myunit = u.def_unit("FOOBAR", u.erg / u.Hz)
+    myunit = u.def_unit("FOOBARa", u.erg / u.Hz)
     assert myunit.to_string('fits') == 'erg Hz-1'
     myunit2 = myunit * u.bit ** 3
     assert myunit2.to_string('fits') == 'bit3 erg Hz-1'
@@ -167,7 +181,7 @@ def test_flatten_to_known():
 
 @raises(ValueError)
 def test_flatten_impossible():
-    myunit = u.def_unit("FOOBAR")
+    myunit = u.def_unit("FOOBARb")
     myunit.to_string('fits')
 
 
@@ -176,3 +190,12 @@ def test_console_out():
     Issue #436.
     """
     u.Jy.decompose().to_string('console')
+
+
+def test_scale_effectively_unity():
+    """Scale just off unity at machine precision level is OK.
+    Ensures #748 does not recur
+    """
+    a = (3. * u.N).cgs
+    assert is_effectively_unity(a.unit.scale)
+    assert len(a.__repr__().split()) == 3
