@@ -78,6 +78,7 @@ def _is_inside(path, parent_path):
     return os.path.abspath(path).startswith(os.path.abspath(parent_path)) \
         or os.path.realpath(path).startswith(os.path.realpath(parent_path))
 
+
 @contextlib.contextmanager
 def get_readable_fileobj(name_or_obj, encoding=None, cache=False):
     """
@@ -122,6 +123,7 @@ def get_readable_fileobj(name_or_obj, encoding=None, cache=False):
     cache : bool, optional
         Whether to cache the contents of remote URLs.
     """
+
     import tempfile
 
     # close_fds is a list of file handles created by this function
@@ -335,7 +337,7 @@ def get_pkg_data_fileobj(data_name, encoding=None, cache=True):
     This will retrieve a data file and its contents for the `astropy.wcs`
     tests::
 
-        from astropy.config import get_pkg_data_fileobj
+        from astropy.utils.data import get_pkg_data_fileobj
 
         with get_pkg_data_fileobj('data/3d_cd.hdr') as fobj:
             fcontents = fobj.read()
@@ -345,7 +347,7 @@ def get_pkg_data_fileobj(data_name, encoding=None, cache=True):
     source distribution.  It will also save the file locally so the
     next time it is accessed it won't need to be downloaded.::
 
-        from astropy.config import get_pkg_data_fileobj
+        from astropy.utils.data import get_pkg_data_fileobj
 
         with get_pkg_data_fileobj('standards/vega.fits') as fobj:
             fcontents = fobj.read()
@@ -420,7 +422,7 @@ def get_pkg_data_filename(data_name):
     This will retrieve the contents of the data file for the `astropy.wcs`
     tests::
 
-        from astropy.config import get_pkg_data_filename
+        from astropy.utils.data import get_pkg_data_filename
 
         fn = get_pkg_data_filename('data/3d_cd.hdr')
         with open(fn) as f:
@@ -430,7 +432,7 @@ def get_pkg_data_filename(data_name):
     This retrieves a data file by hash either locally or from the astropy data
     server::
 
-        from astropy.config import get_pkg_data_filename
+        from astropy.utils.data import get_pkg_data_filename
 
         fn = get_pkg_data_filename('hash/da34a7b07ef153eede67387bf950bb32')
         with open(fn) as f:
@@ -442,6 +444,8 @@ def get_pkg_data_filename(data_name):
     get_pkg_data_fileobj : returns a file-like object with the data
     """
 
+    data_name = os.path.normpath(data_name)
+
     if data_name.startswith('hash/'):
         # first try looking for a local version if a hash is specified
         hashfn = _find_hash_fn(data_name[5:])
@@ -561,7 +565,7 @@ def get_pkg_data_filenames(datadir, pattern='*'):
     This will retrieve the contents of the data file for the `astropy.wcs`
     tests::
 
-        from astropy.config import get_pkg_data_filenames
+        from astropy.utils.data import get_pkg_data_filenames
 
         for fn in get_pkg_data_filename('maps', '*.hdr'):
             with open(fn) as f:
@@ -632,7 +636,7 @@ def get_pkg_data_fileobjs(datadir, pattern='*', encoding=None):
     This will retrieve the contents of the data file for the `astropy.wcs`
     tests::
 
-        from astropy.config import get_pkg_data_filenames
+        from astropy.utils.data import get_pkg_data_filenames
 
         for fd in get_pkg_data_filename('maps', '*.hdr'):
             fcontents = fd.read()
@@ -690,7 +694,18 @@ def _find_pkg_data_path(data_name):
     if module is None:
         # not called from inside an astropy package.  So just pass name through
         return data_name
-    rootpkgname = module.__package__.split('.')[0]
+
+    if not hasattr(module, '__package__') or not module.__package__:
+        # The __package__ attribute may be missing or set to None; see PEP-366,
+        # also astropy issue #1256
+        if '.' in module.__name__:
+            pkgname = module.__name__.rpartition('.')[0]
+        else:
+            pkgname = module.__name__
+    else:
+        pkgname = module.__package__
+
+    rootpkgname = pkgname.partition('.')[0]
 
     rootpkg = __import__(rootpkgname)
 
@@ -786,9 +801,20 @@ def download_file(remote_url, cache=False):
         The URL of the file to download
     cache : bool, optional
         Whether to use the cache
+
+    Returns
+    -------
+    local_path : str
+        Returns the local path that the file was download to.
+
+    Raises
+    ------
+    `urllib2.URLError`
+        Whenever there's a problem getting the remote file.
     """
 
     import hashlib
+    import socket
     from contextlib import closing
     from tempfile import NamedTemporaryFile, gettempdir
     from shutil import move
@@ -877,6 +903,12 @@ def download_file(remote_url, cache=False):
             e.reason.strerror = e.reason.strerror + '. requested URL: ' + remote_url
             e.reason.args = (e.reason.errno, e.reason.strerror)
         raise e
+    except socket.timeout as e:
+        # this isn't supposed to happen, but occasionally a socket.timeout gets
+        # through.  It's supposed to be caught in `urrlib2` and raised in this
+        # way, but for some reason in mysterious circumstances it doesn't. So
+        # we'll just re-raise it here instead
+        raise urllib2.URLError(e)
 
     return local_path
 
