@@ -200,15 +200,6 @@ def find_mod_objs(modname, onlylocals=False):
 
 def isiterable(obj):
     """Returns `True` if the given object is iterable."""
-    from numpy import ndarray
-
-    # Numpy arrays are in collections.Iterable no matter what, but if you
-    # attempt to iterate over a 0-d array, it throws a TypeError.
-    if isinstance(obj, ndarray) and len(obj.shape) == 0:
-        return False
-
-    if isinstance(obj, collections.Iterable):
-        return True
 
     try:
         iter(obj)
@@ -267,9 +258,16 @@ class lazyproperty(object):
             return obj.__dict__[key]
 
     def __set__(self, obj, val):
+        obj_dict = obj.__dict__
+        func_name = self._fget.func_name
         if self._fset:
-            self._fset(obj, val)
-        obj.__dict__[self._fget.func_name] = val
+            ret = self._fset(obj, val)
+            if ret is not None and obj_dict.get(func_name) is ret:
+                # By returning the value set the setter signals that it took
+                # over setting the value in obj.__dict__; this mechanism allows
+                # it to override the input value
+                return
+        obj_dict[func_name] = val
 
     def __delete__(self, obj):
         if self._fdel:
