@@ -1,5 +1,4 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-import glob
 import os
 import sys
 import warnings
@@ -12,6 +11,7 @@ from ...tests.helper import raises
 from ... import wcs
 from ...utils.data import (
     get_pkg_data_filenames, get_pkg_data_contents, get_pkg_data_filename)
+from ...utils.misc import NumpyRNGContext
 from ...tests.helper import pytest
 
 
@@ -248,9 +248,10 @@ def test_backward_compatible():
     fits = get_pkg_data_filename('data/sip.fits')
     w = wcs.WCS(fits)
 
-    data = np.random.rand(100, 2)
-    assert np.all(w.wcs_pix2world(data, 0) == w.wcs_pix2sky(data, 0))
-    assert np.all(w.wcs_world2pix(data, 0) == w.wcs_sky2pix(data, 0))
+    with NumpyRNGContext(123456789):
+        data = np.random.rand(100, 2)
+        assert np.all(w.wcs_pix2world(data, 0) == w.wcs_pix2sky(data, 0))
+        assert np.all(w.wcs_world2pix(data, 0) == w.wcs_sky2pix(data, 0))
 
 
 def test_dict_init():
@@ -290,8 +291,9 @@ def test_extra_kwarg():
     Issue #444
     """
     w = wcs.WCS()
-    data = np.random.rand(100, 2)
-    w.wcs_pix2sky(data, origin=1)
+    with NumpyRNGContext(123456789):
+        data = np.random.rand(100, 2)
+        w.wcs_pix2sky(data, origin=1)
 
 
 def test_3d_shapes():
@@ -299,12 +301,13 @@ def test_3d_shapes():
     Issue #444
     """
     w = wcs.WCS(naxis=3)
-    data = np.random.rand(100, 3)
-    result = w.wcs_pix2sky(data, 1)
-    assert result.shape == (100, 3)
-    result = w.wcs_pix2sky(
-        data[..., 0], data[..., 1], data[..., 2], 1)
-    assert len(result) == 3
+    with NumpyRNGContext(123456789):
+        data = np.random.rand(100, 3)
+        result = w.wcs_pix2sky(data, 1)
+        assert result.shape == (100, 3)
+        result = w.wcs_pix2sky(
+            data[..., 0], data[..., 1], data[..., 2], 1)
+        assert len(result) == 3
 
 
 def test_preserve_shape():
@@ -350,6 +353,32 @@ def test_shape_mismatch():
         xp, yp = w.wcs_world2pix(x, y, 1)
     assert exc.value.args[0] == "Coordinate arrays are not broadcastable to each other"
 
+    # There are some ambiguities that need to be worked around when
+    # naxis == 1
+    w = wcs.WCS(naxis=1)
+
+    x = np.random.random((42, 1))
+    xw = w.wcs_pix2world(x, 1)
+    assert xw.shape == (42, 1)
+
+    x = np.random.random((42,))
+    xw, = w.wcs_pix2world(x, 1)
+    assert xw.shape == (42,)
+
+
+def test_invalid_shape():
+    # Issue #1395
+    w = wcs.WCS(naxis=2)
+
+    xy = np.random.random((2, 3))
+    with pytest.raises(ValueError) as exc:
+        xy2 = w.wcs_pix2world(xy, 1)
+
+    xy = np.random.random((2, 1))
+    with pytest.raises(ValueError) as exc:
+        xy2 = w.wcs_pix2world(xy, 1)
+
+
 @raises(wcs._wcs.InvalidTransformError)
 def test_find_all_wcs_crash():
     """
@@ -359,3 +388,16 @@ def test_find_all_wcs_crash():
 
     with fits.open(get_pkg_data_filename("data/too_many_pv.hdr")) as hdulist:
         wcses = wcs.find_all_wcs(hdulist[0].header)
+
+
+def test_to_header_string():
+    header_string = """
+    WCSAXES =                    2 / Number of coordinate axes                      CRPIX1  =                    0 / Pixel coordinate of reference point            CRPIX2  =                    0 / Pixel coordinate of reference point            CDELT1  =                    1 / Coordinate increment at reference point        CDELT2  =                    1 / Coordinate increment at reference point        CRVAL1  =                    0 / Coordinate value at reference point            CRVAL2  =                    0 / Coordinate value at reference point            LATPOLE =                   90 / [deg] Native latitude of celestial pole        RESTFRQ =                    0 / [Hz] Line rest frequency                       RESTWAV =                    0 / [Hz] Line rest wavelength                      END"""
+
+    w = wcs.WCS()
+    assert w.to_header_string().strip() == header_string.strip()
+
+
+def test_to_fits():
+    w = wcs.WCS()
+    w.to_fits()
