@@ -62,7 +62,7 @@ else:  # pragma: py2
 
 __all__ = ['FITSFixedWarning', 'WCS', 'find_all_wcs',
            'DistortionLookupTable', 'Sip', 'Tabprm', 'UnitConverter',
-           'Wcsprm']
+           'Wcsprm', 'WCSBase']
 
 
 if _wcs is not None:
@@ -275,7 +275,12 @@ class WCS(WCSBase):
             keysel_flags = _parse_keysel(keysel)
 
             if isinstance(header, string_types):
-                if os.path.exists(header):
+                try:
+                    is_path = os.path.exists(header)
+                except (IOError, ValueError):
+                    is_path = False
+
+                if is_path:
                     if fobj is not None:
                         raise ValueError(
                             "Can not provide both a FITS filename to "
@@ -845,21 +850,50 @@ naxis kwarg.
             raise TypeError("Unexpected keyword argument {0!r}".format(
                 kwargs.keys()[0]))
 
-        if len(args) == 2:
-            xy, origin = args
+        def _return_list_of_arrays(axes, origin):
             try:
-                xy = np.asarray(xy)
-                origin = int(origin)
-            except:
-                raise TypeError(
-                    "When providing two arguments, they must be "
-                    "(coords[N][{0}], origin)".format(self.naxis))
+                axes = np.broadcast_arrays(*axes)
+            except ValueError:
+                raise ValueError(
+                    "Coordinate arrays are not broadcastable to each other")
+
+            xy = np.hstack([x.reshape((x.size, 1)) for x in axes])
+
+            if ra_dec_order and sky == 'input':
+                xy = self._denormalize_sky(xy)
+            output = func(xy, origin)
+            if ra_dec_order and sky == 'output':
+                output = self._normalize_sky_output(output)
+                return (output[:, 0].reshape(axes[0].shape),
+                        output[:, 1].reshape(axes[0].shape))
+            return [output[:, i].reshape(axes[0].shape)
+                    for i in range(output.shape[1])]
+
+        def _return_single_array(xy, origin):
+            if xy.shape[-1] != self.naxis:
+                raise ValueError(
+                    "When providing two arguments, the array must be "
+                    "of shape (N, {0})".format(self.naxis))
             if ra_dec_order and sky == 'input':
                 xy = self._denormalize_sky(xy)
             result = func(xy, origin)
             if ra_dec_order and sky == 'output':
                 result = self._normalize_sky(result)
             return result
+
+        if len(args) == 2:
+            try:
+                xy, origin = args
+                xy = np.asarray(xy)
+                origin = int(origin)
+            except:
+                raise TypeError(
+                    "When providing two arguments, they must be "
+                    "(coords[N][{0}], origin)".format(self.naxis))
+            if self.naxis == 1 and len(xy.shape) == 1:
+                return _return_list_of_arrays([xy], origin)
+            return _return_single_array(xy, origin)
+
         elif len(args) == self.naxis + 1:
             axes = args[:-1]
             origin = args[-1]
@@ -871,27 +905,14 @@ naxis kwarg.
                     "When providing more than two arguments, they must be " +
                     "a 1-D array for each axis, followed by an origin.")
 
-            try:
-                axes = np.broadcast_arrays(*axes)
-            except ValueError:
-                raise ValueError(
-                    "Coordinate arrays are not broadcastable to each other")
-
-            xy = np.hstack([x.reshape((x.size, 1)) for x in axes])
-
-            if ra_dec_order and sky == 'input':
-                xy = self._denormalize_sky(xy)
-            sky = func(xy, origin)
-            if ra_dec_order and sky == 'output':
-                sky = self._normalize_sky_output(sky)
-                return (sky[:, 0].reshape(axes[0].shape),
-                        sky[:, 1].reshape(axes[0].shape))
-            return [sky[:, i].reshape(axes[0].shape)
-                    for i in range(sky.shape[1])]
+            return _return_list_of_arrays(axes, origin)
 
         raise TypeError(
-            "Expected 2 or {0} arguments, {0} given".format(
-                self.naxis + 1, len(args)))
+            "WCS projection has {0} dimensions, so expected 2 (an Nx{0} array "
+            "and the origin argument) or {1} arguments (the position in each "
+            "dimension, and the origin argument). Instead, {2} arguments were "
+            "given.".format(
+                self.naxis, self.naxis + 1, len(args)))
 
     def all_pix2world(self, *args, **kwargs):
         return self._array_converter(
